#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2014, Chris Hoffman <choffman@chathamfinancial.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r'''
---
module: win_group
short_description: Add and remove local groups
description:
  - Add and remove local groups.
  - Adds and removes members of local groups.
  - For non-Windows targets, please use the M(ansible.builtin.group) module instead.
options:
  name:
    description:
      - Name of the group.
    type: str
    required: yes
  description:
    description:
      - Description of the group.
      - Set to an empty string C("") to unset the description.
    type: str
  members:
    description:
      - The members of the group to set.
      - The value is a dictionary that contains 3 keys, I(add), I(remove),
        or I(set).
      - Each subkey value is a list of users or domain groups to add, remove,
        or set respectively.
      - The members can either be the username in the form of C(SERVER\user),
        C(DOMAIN\user), C(.\user) to represent a local user, a UPN
        C(user@DOMAIN.COM), or a security identifier C(S-1-5-....).
      - A local group member cannot be another local group, it must be either a
        local user, domain user, or a domain group.
      - The I(add) and I(remove) keys can be set together but I(set) can only
        be set by itself.
    type: dict
    version_added: 2.7.0
    suboptions:
      add:
        description:
          - The members to add to the group.
          - This will add the members without removing any existing members
            not listed.
        default: []
        type: list
        elements: str
      remove:
        description:
          - The members to remove.
          - This will remove the members from the group without removing any
            existing members not listed.
        default: []
        type: list
        elements: str
      set:
        description:
          - The members to set the group to.
          - This will replace the existing membership with the users provided
            in this value.
          - Can be set to C([]) to clear all members from the group.
        type: list
        elements: str
  state:
    description:
      - Create or remove the group.
    type: str
    choices: [ absent, present ]
    default: present
seealso:
  - module: ansible.builtin.group
  - module: community.windows.win_domain_group
author:
  - Chris Hoffman (@chrishoffman)
'''

EXAMPLES = r'''
- name: Create a new group
  ansible.windows.win_group:
    name: deploy
    description: Deploy Group
    state: present

- name: Remove a group
  ansible.windows.win_group:
    name: deploy
    state: absent

- name: Remove the group description
  ansible.windows.win_group:
    name: MyGroup
    description: ""
    state: present

- name: Add a user to a group
  ansible.windows.win_group:
    name: deploy
    members:
      add:
        - .\LocalUser1
        - LocalUser2
        - DOMAIN\User
        - user@DOMAIN.COM
        - S-1-5-0-10-204-0189-500
    state: present

- name: Remove a user from a group
  ansible.windows.win_group:
    name: deploy
    members:
      remove:
        - .\LocalUser1

- name: Set the members of a group
  ansible.windows.win_group:
    name: deploy
    members:
      set:
        - .\LocalUser1
        - LocalUser2
        - DOMAIN\User

- name: Remove all members of a group
  ansible.windows.win_group:
    name: deploy
    members:
      set: []
'''

RETURN = r'''
sid:
  description:
    - The Security Identifier (SID) of the group being managed.
    - If a new group was created in check mode, the SID will be C(S-1-5-0000).
    - When the group is not present, the SID will be C(None).
  returned: always
  type: str
  sample: S-1-5-21-2528685370-1724360342-165486190-1208
'''



# Copyright (c) 2018, IBM CORPORATION
# Author(s): Tzur Eliyahu <tzure@il.ibm.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this doc fragment is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations


class ModuleDocFragment:
    # ibm_storage documentation fragment
    DOCUMENTATION = r"""
options:
  username:
    description:
      - Management user on the Spectrum Accelerate storage system.
    type: str
    required: true
  password:
    description:
      - Password for username on the Spectrum Accelerate storage system.
    type: str
    required: true
  endpoints:
    description:
      - The hostname or management IP of Spectrum Accelerate storage system.
    type: str
    required: true
notes:
  - This module requires pyxcli python library. Use C(pip install pyxcli) in order to get pyxcli.
requirements:
  - pyxcli
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2023, Gaspard Micol (@gmicol) <gmicol@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: aci_l3out_eigrp_interface_profile
short_description: Manage Layer 3 Outside (L3Out) EIGRP interface profile (eigrp:IfP)
description:
- Manage L3Out logical interface profile EIGRP policies on Cisco ACI fabrics.
options:
  tenant:
    description:
    - The name of an existing tenant.
    type: str
    aliases: [ tenant_name ]
  l3out:
    description:
    - The name of an existing L3Out.
    type: str
    aliases: [ l3out_name ]
  node_profile:
    description:
    - The name of the node profile.
    type: str
    aliases: [ node_profile_name, logical_node ]
  interface_profile:
    description:
    - The name of an existing interface profile.
    type: str
    aliases: [ interface_profile_name, logical_interface ]
  eigrp_policy:
    description:
    - The name of an existing EIGRP interface policy.
    type: str
    aliases: [ name, eigrp_policy_name ]
  eigrp_keychain_policy:
    description:
    - The name of an existing EIGRP keychain policy.
    - Pass an empty string to disable Authentification.
    type: str
    aliases: [ keychain_policy, keychain_policy_name  ]
  description:
    description:
    - The description of the EIGRP interface profile.
    type: str
    aliases: [ descr ]
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation
- cisco.aci.owner

notes:
- The C(tenant), C(l3out), C(node_profile), C(interface_profile) and C(eigrp_policy) must exist before using this module in your playbook.
  The M(cisco.aci.aci_tenant), M(cisco.aci.aci_l3out), M(cisco.aci.aci_l3out_logical_node_profile), M(cisco.aci.aci_l3out_logical_interface_profile)
  and M(cisco.aci.aci_interface_policy_eigrp) can be used for this.
- if C(eigrp_keychain_policy) is used, it must exist before using this module in your playbook.
  The M(cisco.aci.aci_keychain_policy) can be used for this.
seealso:
- module: cisco.aci.aci_tenant
- module: cisco.aci.aci_l3out
- module: cisco.aci.aci_l3out_logical_node_profile
- module: cisco.aci.aci_l3out_logical_interface_profile
- module: cisco.aci.aci_interface_policy_eigrp
- module: cisco.aci.aci_keychain_policy
- name: APIC Management Information Model reference
  description: More information about the internal APIC classes
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Gaspard Micol (@gmicol)
"""

EXAMPLES = r"""
- name: Add a new interface profile EIGRP policy
  cisco.aci.aci_l3out_eigrp_interface_profile:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    l3out: my_l3out
    node_profile: my_node_profile
    interface_profile: my_interface_profile
    eigrp_policy: my_eigrp_interface_policy
    state: present
  delegate_to: localhost

- name: Add a new interface profile EIGRP policy with authentication
  cisco.aci.aci_l3out_eigrp_interface_profile:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    l3out: my_l3out
    node_profile: my_node_profile
    interface_profile: my_interface_profile
    eigrp_policy: my_eigrp_interface_policy
    eigrp_keychain_policy: my_keychain_policy
    state: present
  delegate_to: localhost

- name: Disable authentification from an interface profile EIGRP policy
  cisco.aci.aci_l3out_eigrp_interface_profile:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    l3out: my_l3out
    node_profile: my_node_profile
    interface_profile: my_interface_profile
    eigrp_policy: my_eigrp_interface_policy
    eigrp_keychain_policy: ""
    state: present
  delegate_to: localhost

- name: Delete an interface profile EIGRP policy
  cisco.aci.aci_l3out_eigrp_interface_profile:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    l3out: my_l3out
    node_profile: my_node_profile
    interface_profile: my_interface_profile
    eigrp_policy: my_eigrp_interface_policy
    state: absent
  delegate_to: localhost

- name: Query an interface profile EIGRP policy
  cisco.aci.aci_l3out_eigrp_interface_profile:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    l3out: my_l3out
    node_profile: my_node_profile
    interface_profile: my_interface_profile
    eigrp_policy: my_eigrp_interface_policy
    state: query
  delegate_to: localhost
  register: query_result
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec, aci_owner_spec


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(aci_owner_spec())
    argument_spec.update(
        tenant=dict(type="str", aliases=["tenant_name"]),
        l3out=dict(type="str", aliases=["l3out_name"]),
        node_profile=dict(type="str", aliases=["node_profile_name", "logical_node"]),
        interface_profile=dict(type="str", aliases=["interface_profile_name", "logical_interface"]),
        eigrp_policy=dict(type="str", aliases=["name", "eigrp_policy_name"]),
        eigrp_keychain_policy=dict(type="str", aliases=["keychain_policy", "keychain_policy_name"], no_log=False),
        description=dict(type="str", aliases=["descr"]),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["tenant", "l3out", "node_profile", "interface_profile"]],
            ["state", "present", ["tenant", "l3out", "node_profile", "interface_profile", "eigrp_policy"]],
        ],
    )

    tenant = module.params.get("tenant")
    l3out = module.params.get("l3out")
    node_profile = module.params.get("node_profile")
    interface_profile = module.params.get("interface_profile")
    eigrp_policy = module.params.get("eigrp_policy")
    eigrp_keychain_policy = module.params.get("eigrp_keychain_policy")
    description = module.params.get("description")
    state = module.params.get("state")

    aci = ACIModule(module)

    child_classes = ["eigrpRsIfPol", "eigrpAuthIfP"]

    aci.construct_url(
        root_class=dict(
            aci_class="fvTenant",
            aci_rn="tn-{0}".format(tenant),
            module_object=tenant,
            target_filter={"name": tenant},
        ),
        subclass_1=dict(
            aci_class="l3extOut",
            aci_rn="out-{0}".format(l3out),
            module_object=l3out,
            target_filter={"name": l3out},
        ),
        subclass_2=dict(
            aci_class="l3extLNodeP",
            aci_rn="lnodep-{0}".format(node_profile),
            module_object=node_profile,
            target_filter={"name": node_profile},
        ),
        subclass_3=dict(
            aci_class="l3extLIfP",
            aci_rn="lifp-[{0}]".format(interface_profile),
            module_object=interface_profile,
            target_filter={"name": interface_profile},
        ),
        subclass_4=dict(
            aci_class="eigrpIfP",
            aci_rn="eigrpIfP",
            module_object=interface_profile,
            target_filter={"name": interface_profile},
        ),
        child_classes=child_classes,
    )

    aci.get_existing()

    if state == "present":
        child_configs = [dict(eigrpRsIfPol=dict(attributes=dict(tnEigrpIfPolName=eigrp_policy)))]

        if eigrp_keychain_policy is not None:
            if eigrp_keychain_policy == "" and isinstance(aci.existing, list) and len(aci.existing) > 0:
                for child in aci.existing[0].get("eigrpIfP", {}).get("children", {}):
                    if child.get("eigrpAuthIfP"):
                        child_configs.append(
                            dict(
                                eigrpAuthIfP=dict(
                                    attributes=dict(status="deleted"),
                                ),
                            )
                        )
            elif eigrp_keychain_policy != "":
                child_configs.append(
                    dict(
                        eigrpAuthIfP=dict(
                            attributes=dict(),
                            children=[
                                dict(
                                    eigrpRsKeyChainPol=dict(
                                        attributes=dict(
                                            tnFvKeyChainPolName=eigrp_keychain_policy,
                                        ),
                                    )
                                )
                            ],
                        )
                    )
                )

        aci.payload(
            aci_class="eigrpIfP",
            class_config=dict(
                descr=description,
            ),
            child_configs=child_configs,
        )

        aci.get_diff(aci_class="eigrpIfP")

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2021 Paul Aiton, < @paultaiton >
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_managementgroup_info

version_added: "1.5.0"

short_description: Get Azure Management Group facts

description:
    - Get facts for a specific Management Group or all Management Groups.

options:
    name:
        description:
            - Limit results to a specific management group by name.
            - Mutually exclusive with I(id).
        aliases:
            - management_group_name
        type: str
    id:
        description:
            - Limit results to a specific management group by id.
            - Mutually exclusive with I(name).
        type: str
    flatten:
        description:
            - If c(True) then child management_groups and subscriptions will be copied to the root
              of the management_groups and subscriptions return list respectively.
            - By default c(False), child elements will only apear in the nested complex.
            - Option only matters when I(children) is c(True), and will otherwise be silently ignored.
        type: bool
        default: False
    children:
        description:
            - If c(False), then only I(name) or I(id) group will be fetched, or only the list of root groups.
            - If c(True), then the children groups will also be returned.
        type: bool
        default: False
    recurse:
        description:
            - By default, c(False), only the direct children are returned if I(children) is c(True).
            - If c(True), then all descendants of the heirarchy are returned.
            - Option only matters when I(children) is c(True), and will otherwise be silently ignored.
        type: bool
        default: False

notes:
    - azure_rm_managementgroup_info - The roles assigned to the principal executing the playbook will determine what is
      a root management_group. You may also be able to request the details of a parent management group, but unable to
      fetch that group. It is highly recommended that if I(children) is set c(True) that specific management groups are
      requested since a list of all groups will require an additional Azure API call for each returned group.

seealso:
    - module: azure.azcollection.azure_rm_subscription_info
      description: module to look up more in depth information on subscriptions; for example tags.
    - module: azure.azcollection.azure_rm_roleassignment_info
      description: module to look up RBAC role assignments, which can use management group id as scope.

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Paul Aiton (@paultaiton)
'''

EXAMPLES = '''
- name: Get facts for all root management groups for authenticated principal
  azure_rm_managementgroup_info:

- name: Get facts for one management group by id with direct children
  azure_rm_managementgroup_info:
    id: /providers/Microsoft.Management/managementGroups/contoso-group
    children: true

- name: Get facts for one management group by name with all children, flattened into top list
  azure_rm_managementgroup_info:
    name: "contoso-group"
    children: true
    recurse: true
    flatten: true
'''

RETURN = '''
management_groups:
    description:
        - List of Management Group dicts.
    returned: always
    type: list
    contains:
        display_name:
            description: Management Group display name.
            returned: always
            type: str
            sample: "My Management Group"
        id:
            description: Management Group fully qualified id.
            returned: always
            type: str
            sample: "/providers/Microsoft.Management/managementGroups/group-name"
        name:
            description: Management Group display name.
            returned: always
            type: str
            sample: group-name
        tenant_id:
            description: Management Group tenant id
            returned: always
            type: str
            sample: "00000000-0000-0000-0000-000000000000"
        type:
            description: Management Group type
            returned: always
            type: str
            sample: "/providers/Microsoft.Management/managementGroups"
        children:
            description: Child management groups or subscriptions.
            returned: if I(children) is c(True)
            type: list
            sample: Nested list of children. Same as top groups, but without tenant_id.
subscriptions:
    description:
        - List of subscription objects.
    returned: if I(children) and I(flatten) are both c(True)
    type: list
    contains:
        display_name:
            description: subscription display name.
            returned: always
            type: str
            sample: "some-subscription-name"
        id:
            description: subscription fully qualified id.
            returned: always
            type: str
            sample: "/subscriptions/00000000-0000-0000-0000-feedc0ffee000000"
        subscription_id:
            description: subscription guid.
            returned: always
            type: str
            sample: "00000000-0000-0000-0000-feedc0ffee000000"
        type:
            description: Management Group type
            returned: always
            type: str
            sample: "/subscriptions"
'''


from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase


class AzureRMManagementGroupInfo(AzureRMModuleBase):
    def __init__(self):
        self.module_arg_spec = dict(
            children=dict(type='bool', default=False),
            flatten=dict(type='bool', default=False),
            id=dict(type='str'),
            name=dict(type='str', aliases=['management_group_name']),
            recurse=dict(type='bool', default=False)
        )

        self.results = dict(
            changed=False,
            management_groups=[]
        )

        self.children = None
        self.flatten = None
        self.id = None
        self.name = None
        self.recurse = None

        mutually_exclusive = [['name', 'id']]

        super(AzureRMManagementGroupInfo, self).__init__(self.module_arg_spec,
                                                         supports_check_mode=True,
                                                         supports_tags=False,
                                                         mutually_exclusive=mutually_exclusive,
                                                         facts_module=True)

    def exec_module(self, **kwargs):
        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        response = []

        if self.name or self.id:
            response = [self.get_item()]
        else:
            response = self.list_items()

        if self.flatten and self.children:
            self.results['subscriptions'] = []
            for group in response:
                new_groups = []
                new_subscriptions = []
                self.results['management_groups'].append(group)
                new_groups, new_subscriptions = self.flatten_group(group)
                self.results['management_groups'] += new_groups
                self.results['subscriptions'] += new_subscriptions
        else:
            self.results['management_groups'] = response

        return self.results

    def get_item(self, mg_name=None):
        if not mg_name:
            # The parameter to SDK's management_groups.get(group_id) is not correct,
            # it only works with a bare name value, and not the fqid.
            if self.id and not self.name:
                mg_name = self.id.split('/')[-1]
            else:
                mg_name = self.name

        expand = 'children' if self.children else None
        try:
            response = self.management_groups_client.management_groups.get(group_id=mg_name,
                                                                           expand=expand,
                                                                           recurse=self.recurse)
        except Exception as e:
            self.log('No Management group {0} found. msg: {1}'.format(mg_name, e))
            response = []

        return self.to_dict(response)

    def list_items(self):
        self.log('List all management groups.')

        results = []
        response = []

        try:
            response = self.management_groups_client.management_groups.list()
        except Exception as e:
            self.log('No Management groups found.msg: {0}'.format(e))
            pass  # default to response of an empty list

        if self.children:
            # list method cannot return children, so we must iterate over root management groups to
            # get each one individually.
            results = [self.get_item(mg_name=item.name) for item in response]
        else:
            results = [self.to_dict(item) for item in response]

        return results

    def to_dict(self, azure_object):
        if not azure_object:
            return []
        if azure_object.type == 'Microsoft.Management/managementGroups':
            return_dict = dict(
                display_name=azure_object.display_name,
                id=azure_object.id,
                name=azure_object.name,
                type=azure_object.type
            )

            # If group has no children, then property will be set to None type.
            # We want an empty list so that it can be used in loops without issue.
            if self.children and azure_object.as_dict().get('children'):
                return_dict['children'] = [self.to_dict(item) for item in azure_object.children]
            elif self.children:
                return_dict['children'] = []

            if azure_object.as_dict().get('details', {}).get('parent'):
                parent_dict = azure_object.as_dict().get('details', {}).get('parent')
                return_dict['parent'] = dict(
                    display_name=parent_dict.get('display_name'),
                    id=parent_dict.get('id'),
                    name=parent_dict.get('name')
                )

        elif azure_object.type == '/subscriptions':
            return_dict = dict(
                display_name=azure_object.display_name,
                id=azure_object.id,
                subscription_id=azure_object.name,
                type=azure_object.type
            )
        else:
            # In theory if the Azure API is updated to include another child type of management groups,
            # the code here will prevent an exception. But there should be logic added in an update to take
            # care of a new child type of management groups.
            return_dict = dict(
                state="This is an unknown and unexpected object. "
                      + "You should report this as a bug to the ansible-collection/azcollection "
                      + "project on github. Please include the object type in your issue report, "
                      + "and @ the authors of this module. ",
                type=azure_object.as_dict().get('type', None)
            )

        if azure_object.as_dict().get('tenant_id'):
            return_dict['tenant_id'] = azure_object.tenant_id

        return return_dict

    def flatten_group(self, management_group):
        management_group_list = []
        subscription_list = []
        if management_group.get('children'):
            for child in management_group.get('children', []):
                if child.get('type') == '/providers/Microsoft.Management/managementGroups':
                    management_group_list.append(child)
                    new_groups, new_subscriptions = self.flatten_group(child)
                    management_group_list += new_groups
                    subscription_list += new_subscriptions
                elif child.get('type') == '/subscriptions':
                    subscription_list.append(child)
        return management_group_list, subscription_list


def main():
    AzureRMManagementGroupInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: networks_wireless_ssids_hotspot20
short_description: Resource module for networks _wireless _ssids _hotspot20
description:
  - Manage operation update of the resource networks _wireless _ssids _hotspot20.
  - Update the Hotspot 2.0 settings of an SSID.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  domains:
    description: An array of domain names.
    elements: str
    type: list
  enabled:
    description: Whether or not Hotspot 2.0 for this SSID is enabled.
    type: bool
  mccMncs:
    description: An array of MCC/MNC pairs.
    elements: dict
    suboptions:
      mcc:
        description: MCC value.
        type: str
      mnc:
        description: MNC value.
        type: str
    type: list
  naiRealms:
    description: An array of NAI realms.
    elements: dict
    suboptions:
      format:
        description: The format for the realm ('1' or '0').
        type: str
      methods:
        description: An array of EAP methods for the realm.
        elements: dict
        suboptions:
          authenticationTypes:
            description: The authentication types for the method. These should be
              formatted as an object with the EAP method category in camelcase as
              the key and the list of types as the value (nonEapInnerAuthentication
              Reserved, PAP, CHAP, MSCHAP, MSCHAPV2; eapInnerAuthentication EAP-TLS,
              EAP-SIM, EAP-AKA, EAP-TTLS with MSCHAPv2; credentials SIM, USIM, NFC
              Secure Element, Hardware Token, Softoken, Certificate, username/password,
              none, Reserved, Vendor Specific; tunneledEapMethodCredentials SIM, USIM,
              NFC Secure Element, Hardware Token, Softoken, Certificate, username/password,
              Reserved, Anonymous, Vendor Specific).
            suboptions:
              credentials:
                description: An array of credentials.
                elements: str
                type: list
              eapInnerAuthentication:
                description: An array of EAP inner authentication types.
                elements: str
                type: list
              nonEapInnerAuthentication:
                description: An array of non-EAP inner authentication types.
                elements: str
                type: list
              tunneledEapMethodCredentials:
                description: An array of tunneled EAP method credentials.
                elements: str
                type: list
            type: dict
          id:
            description: ID of method.
            type: str
        type: list
      realm:
        description: The name of the realm.
        type: str
    type: list
  networkAccessType:
    description: The network type of this SSID ('Private network', 'Private network
      with guest access', 'Chargeable public network', 'Free public network', 'Personal
      device network', 'Emergency services only network', 'Test or experimental',
      'Wildcard').
    type: str
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
  number:
    description: Number path parameter.
    type: str
  operator:
    description: Operator settings for this SSID.
    suboptions:
      name:
        description: Operator name.
        type: str
    type: dict
  roamConsortOis:
    description: An array of roaming consortium OIs (hexadecimal number 3-5 octets
      in length).
    elements: str
    type: list
  venue:
    description: Venue settings for this SSID.
    suboptions:
      name:
        description: Venue name.
        type: str
      type:
        description: Venue type ('Unspecified', 'Unspecified Assembly', 'Arena', 'Stadium',
          'Passenger Terminal', 'Amphitheater', 'Amusement Park', 'Place of Worship',
          'Convention Center', 'Library', 'Museum', 'Restaurant', 'Theater', 'Bar',
          'Coffee Shop', 'Zoo or Aquarium', 'Emergency Coordination Center', 'Unspecified
          Business', 'Doctor or Dentist office', 'Bank', 'Fire Station', 'Police Station',
          'Post Office', 'Professional Office', 'Research and Development Facility',
          'Attorney Office', 'Unspecified Educational', 'School, Primary', 'School,
          Secondary', 'University or College', 'Unspecified Factory and Industrial',
          'Factory', 'Unspecified Institutional', 'Hospital', 'Long-Term Care Facility',
          'Alcohol and Drug Rehabilitation Center', 'Group Home', 'Prison or Jail',
          'Unspecified Mercantile', 'Retail Store', 'Grocery Market', 'Automotive
          Service Station', 'Shopping Mall', 'Gas Station', 'Unspecified Residential',
          'Private Residence', 'Hotel or Motel', 'Dormitory', 'Boarding House', 'Unspecified
          Storage', 'Unspecified Utility and Miscellaneous', 'Unspecified Vehicular',
          'Automobile or Truck', 'Airplane', 'Bus', 'Ferry', 'Ship or Boat', 'Train',
          'Motor Bike', 'Unspecified Outdoor', 'Muni-mesh Network', 'City Park', 'Rest
          Area', 'Traffic Control', 'Bus Stop', 'Kiosk').
        type: str
    type: dict
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for wireless updateNetworkWirelessSsidHotspot20
    description: Complete reference of the updateNetworkWirelessSsidHotspot20 API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-network-wireless-ssid-hotspot20
notes:
  - SDK Method used are
    wireless.Wireless.update_network_wireless_ssid_hotspot20,
  - Paths used are
    put /networks/{networkId}/wireless/ssids/{number}/hotspot20,
"""

EXAMPLES = r"""
- name: Update all
  cisco.meraki.networks_wireless_ssids_hotspot20:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    domains:
      - meraki.local
      - domain2.com
    enabled: true
    mccMncs:
      - mcc: '123'
        mnc: '456'
    naiRealms:
      - format: '1'
        methods:
          - authenticationTypes:
              credentials: []
              eapInnerAuthentication:
                - EAP-TTLS with MSCHAPv2
              nonEapInnerAuthentication:
                - MSCHAP
              tunneledEapMethodCredentials: []
            id: '1'
        realm: Realm 1
    networkAccessType: Private network
    networkId: string
    number: string
    operator:
      name: Meraki Product Management
    roamConsortOis:
      - ABC123
      - 456EFG
    venue:
      name: SF Branch
      type: Unspecified Assembly
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "enabled": true,
      "operator": {
        "name": "string"
      },
      "venue": {
        "name": "string",
        "type": "string"
      },
      "networkAccessType": "string",
      "domains": [
        "string"
      ],
      "roamConsortOis": [
        "string"
      ],
      "mccMncs": [
        {
          "mcc": "string",
          "mnc": "string"
        }
      ],
      "naiRealms": [
        {
          "format": "string",
          "name": "string",
          "methods": [
            {
              "id": "string",
              "authenticationTypes": {
                "nonEapInnerAuthentication": [
                  "string"
                ],
                "eapInnerAuthentication": [
                  "string"
                ],
                "credentials": [
                  "string"
                ],
                "tunneledEapMethodCredentials": [
                  "string"
                ]
              }
            }
          ]
        }
      ]
    }
"""



#!/usr/bin/python

# Copyright (c) Ansible project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: github_deploy_key
author: "Ali (@bincyber)"
short_description: Manages deploy keys for GitHub repositories
description:
  - Adds or removes deploy keys for GitHub repositories. Supports authentication using username and password, username and
    password and 2-factor authentication code (OTP), OAuth2 token, or personal access token. Admin rights on the repository
    are required.
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  github_url:
    description:
      - The base URL of the GitHub API.
    type: str
    version_added: '0.2.0'
    default: https://api.github.com
  owner:
    description:
      - The name of the individual account or organization that owns the GitHub repository.
    required: true
    aliases: ['account', 'organization']
    type: str
  repo:
    description:
      - The name of the GitHub repository.
    required: true
    aliases: ['repository']
    type: str
  name:
    description:
      - The name for the deploy key.
    required: true
    aliases: ['title', 'label']
    type: str
  key:
    description:
      - The SSH public key to add to the repository as a deploy key.
    required: true
    type: str
  read_only:
    description:
      - If V(true), the deploy key is only able to read repository contents. Otherwise, the deploy key is able to read and
        write.
    type: bool
    default: true
  state:
    description:
      - The state of the deploy key.
    default: "present"
    choices: ["present", "absent"]
    type: str
  force:
    description:
      - If V(true), forcefully adds the deploy key by deleting any existing deploy key with the same public key or title.
    type: bool
    default: false
  username:
    description:
      - The username to authenticate with. Should not be set when using personal access token.
    type: str
  password:
    description:
      - The password to authenticate with. Alternatively, a personal access token can be used instead of O(username) and O(password)
        combination.
    type: str
  token:
    description:
      - The OAuth2 token or personal access token to authenticate with. Mutually exclusive with O(password).
    type: str
  otp:
    description:
      - The 6 digit One Time Password for 2-Factor Authentication. Required together with O(username) and O(password).
    type: int
notes:
  - "Refer to GitHub's API documentation here: https://developer.github.com/v3/repos/keys/."
"""

EXAMPLES = r"""
- name: Add a new read-only deploy key to a GitHub repository using basic authentication
  community.general.github_deploy_key:
    owner: "johndoe"
    repo: "example"
    name: "new-deploy-key"
    key: "ssh-rsa AAAAB3NzaC1yc2EAAAADAQABAAABAQDAwXxn7kIMNWzcDfou..."
    read_only: true
    username: "johndoe"
    password: "supersecretpassword"

- name: Remove an existing deploy key from a GitHub repository
  community.general.github_deploy_key:
    owner: "johndoe"
    repository: "example"
    name: "new-deploy-key"
    key: "ssh-rsa AAAAB3NzaC1yc2EAAAADAQABAAABAQDAwXxn7kIMNWzcDfou..."
    force: true
    username: "johndoe"
    password: "supersecretpassword"
    state: absent

- name: Add a new deploy key to a GitHub repository, replace an existing key, use an OAuth2 token to authenticate
  community.general.github_deploy_key:
    owner: "johndoe"
    repository: "example"
    name: "new-deploy-key"
    key: "{{ lookup('file', '~/.ssh/github.pub') }}"
    force: true
    token: "ABAQDAwXxn7kIMNWzcDfo..."

- name: Re-add a deploy key to a GitHub repository but with a different name
  community.general.github_deploy_key:
    owner: "johndoe"
    repository: "example"
    name: "replace-deploy-key"
    key: "{{ lookup('file', '~/.ssh/github.pub') }}"
    username: "johndoe"
    password: "supersecretpassword"

- name: Add a new deploy key to a GitHub repository using 2FA
  community.general.github_deploy_key:
    owner: "johndoe"
    repo: "example"
    name: "new-deploy-key-2"
    key: "ssh-rsa AAAAB3NzaC1yc2EAAAADAQABAAABAQDAwXxn7kIMNWzcDfou..."
    username: "johndoe"
    password: "supersecretpassword"
    otp: 123456

- name: Add a read-only deploy key to a repository hosted on GitHub Enterprise
  community.general.github_deploy_key:
    github_url: "https://api.example.com"
    owner: "janedoe"
    repo: "example"
    name: "new-deploy-key"
    key: "ssh-rsa AAAAB3NzaC1yc2EAAAADAQABAAABAQDAwXxn7kIMNWzcDfou..."
    read_only: true
    username: "janedoe"
    password: "supersecretpassword"
"""

RETURN = r"""
msg:
  description: The status message describing what occurred.
  returned: always
  type: str
  sample: "Deploy key added successfully"

http_status_code:
  description: The HTTP status code returned by the GitHub API.
  returned: failed
  type: int
  sample: 400

error:
  description: The error message returned by the GitHub API.
  returned: failed
  type: str
  sample: "key is already in use"

id:
  description: The key identifier assigned by GitHub for the deploy key.
  returned: changed
  type: int
  sample: 24381901
"""

from re import findall

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.urls import fetch_url


class GithubDeployKey:
    def __init__(self, module):
        self.module = module

        self.github_url = self.module.params["github_url"]
        self.name = module.params["name"]
        self.key = module.params["key"]
        self.state = module.params["state"]
        self.read_only = module.params.get("read_only", True)
        self.force = module.params.get("force", False)
        self.username = module.params.get("username", None)
        self.password = module.params.get("password", None)
        self.token = module.params.get("token", None)
        self.otp = module.params.get("otp", None)

    @property
    def url(self):
        owner = self.module.params["owner"]
        repo = self.module.params["repo"]
        return f"{self.github_url}/repos/{owner}/{repo}/keys"

    @property
    def headers(self):
        if self.username is not None and self.password is not None:
            self.module.params["url_username"] = self.username
            self.module.params["url_password"] = self.password
            self.module.params["force_basic_auth"] = True
            if self.otp is not None:
                return {"X-GitHub-OTP": self.otp}
        elif self.token is not None:
            return {"Authorization": f"token {self.token}"}
        else:
            return None

    def paginate(self, url):
        while url:
            resp, info = fetch_url(self.module, url, headers=self.headers, method="GET")

            if info["status"] == 200:
                yield self.module.from_json(resp.read())

                links = {}
                for x, y in findall(r'<([^>]+)>;\s*rel="(\w+)"', info.get("link", "")):
                    links[y] = x

                url = links.get("next")
            else:
                self.handle_error(method="GET", info=info)

    def get_existing_key(self):
        for keys in self.paginate(self.url):
            if keys:
                for i in keys:
                    existing_key_id = str(i["id"])
                    if i["key"].split() == self.key.split()[:2]:
                        return existing_key_id
                    elif i["title"] == self.name and self.force:
                        return existing_key_id
            else:
                return None

    def add_new_key(self):
        request_body = {"title": self.name, "key": self.key, "read_only": self.read_only}

        resp, info = fetch_url(
            self.module,
            self.url,
            data=self.module.jsonify(request_body),
            headers=self.headers,
            method="POST",
            timeout=30,
        )

        status_code = info["status"]

        if status_code == 201:
            response_body = self.module.from_json(resp.read())
            key_id = response_body["id"]
            self.module.exit_json(changed=True, msg="Deploy key successfully added", id=key_id)
        elif status_code == 422:
            # there might be multiple reasons for a 422
            # so we must check if the reason is that the key already exists
            if self.get_existing_key():
                self.module.exit_json(changed=False, msg="Deploy key already exists")
            else:
                self.handle_error(method="POST", info=info)
        else:
            self.handle_error(method="POST", info=info)

    def remove_existing_key(self, key_id):
        resp, info = fetch_url(self.module, f"{self.url}/{key_id}", headers=self.headers, method="DELETE")

        status_code = info["status"]

        if status_code == 204:
            if self.state == "absent":
                self.module.exit_json(changed=True, msg="Deploy key successfully deleted", id=key_id)
        else:
            self.handle_error(method="DELETE", info=info, key_id=key_id)

    def handle_error(self, method, info, key_id=None):
        status_code = info["status"]
        body = info.get("body")
        if body:
            err = self.module.from_json(body)["message"]
        else:
            err = None

        if status_code == 401:
            self.module.fail_json(
                msg=f"Failed to connect to {self.github_url} due to invalid credentials",
                http_status_code=status_code,
                error=err,
            )
        elif status_code == 404:
            self.module.fail_json(msg="GitHub repository does not exist", http_status_code=status_code, error=err)
        else:
            if method == "GET":
                self.module.fail_json(
                    msg="Failed to retrieve existing deploy keys", http_status_code=status_code, error=err
                )
            elif method == "POST":
                self.module.fail_json(msg="Failed to add deploy key", http_status_code=status_code, error=err)
            elif method == "DELETE":
                self.module.fail_json(
                    msg="Failed to delete existing deploy key", id=key_id, http_status_code=status_code, error=err
                )


def main():
    module = AnsibleModule(
        argument_spec=dict(
            github_url=dict(type="str", default="https://api.github.com"),
            owner=dict(required=True, type="str", aliases=["account", "organization"]),
            repo=dict(required=True, type="str", aliases=["repository"]),
            name=dict(required=True, type="str", aliases=["title", "label"]),
            key=dict(required=True, type="str", no_log=False),
            read_only=dict(type="bool", default=True),
            state=dict(default="present", choices=["present", "absent"]),
            force=dict(type="bool", default=False),
            username=dict(type="str"),
            password=dict(type="str", no_log=True),
            otp=dict(type="int", no_log=True),
            token=dict(type="str", no_log=True),
        ),
        mutually_exclusive=[["password", "token"]],
        required_together=[["username", "password"], ["otp", "username", "password"]],
        required_one_of=[["username", "token"]],
        supports_check_mode=True,
    )

    deploy_key = GithubDeployKey(module)

    if module.check_mode:
        key_id = deploy_key.get_existing_key()
        if deploy_key.state == "present" and key_id is None:
            module.exit_json(changed=True)
        elif deploy_key.state == "present" and key_id is not None:
            module.exit_json(changed=False)

    # to forcefully modify an existing key, the existing key must be deleted first
    if deploy_key.state == "absent" or deploy_key.force:
        key_id = deploy_key.get_existing_key()

        if key_id is not None:
            deploy_key.remove_existing_key(key_id)
        elif deploy_key.state == "absent":
            module.exit_json(changed=False, msg="Deploy key does not exist")

    if deploy_key.state == "present":
        deploy_key.add_new_key()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2024 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
The module file for iosxr_vrf_address_family
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
module: iosxr_vrf_address_family
short_description: Resource module to configure VRF Address family.
description:
  - This module configures and manages the attributes of VRF address family on Cisco IOS-XR devices.
version_added: 10.0.0
author: Ruchi Pakhle (@Ruchip16)
notes:
  - Tested against Cisco IOSXR Version 10.0.0
  - This module works with connection C(network_cli). See L(the IOS_XR Platform Options,../network/user_guide/platform_iosxr.html)
  - For more information on using Ansible to manage network devices see the :ref:`Ansible Network Guide <network_guide>`
  - For more information on using Ansible to manage Cisco devices see the `Cisco integration page <https://www.ansible.com/integrations/networks/cisco>`.
options:
  config:
    description: VRF address family configuration.
    type: list
    elements: dict
    suboptions:
      name:
        description: Name of the VRF.
        type: str
        required: true
      address_families:
        description: Enable address family and enter its config mode - AFI/SAFI configuration
        type: list
        elements: dict
        suboptions:
          afi:
            description: Address Family Identifier (AFI)
            type: str
            choices: ['ipv4', 'ipv6']
          safi:
            description: Address Family modifier
            type: str
            choices: [ 'flowspec', 'multicast', 'unicast']
          export:
            description: VRF export
            type: dict
            suboptions:
              route_policy: &route_policy
                description: Use route_policy for export
                type: str
              route_target: &route_target
                description: Specify export route target extended communities.
                type: str
              to:
                description: Export routes to a VRF
                type: dict
                suboptions:
                  default_vrf: &default_vrf
                    description: Export routes to default VRF
                    type: dict
                    suboptions:
                      route_policy: *route_policy
                  vrf:
                    description: Export routes to a VRF
                    type: dict
                    suboptions:
                      allow_imported_vpn:
                        description: Allow export of imported VPN routes to non-default VRF
                        type: bool
          import_config:
            description: VRF import
            type: dict
            suboptions:
              route_policy: *route_policy
              route_target: *route_target
              from_config:
                description: Import routes from a VRF
                type: dict
                suboptions:
                  bridge_domain:
                    description: VRF import
                    type: dict
                    suboptions:
                      advertise_as_vpn: &advertise_as_vpn
                        description: Advertise local EVPN imported routes to PEs
                        type: bool
                  default_vrf: *default_vrf
                  vrf:
                    description: Import routes from a VRF
                    type: dict
                    suboptions:
                      advertise_as_vpn: *advertise_as_vpn
          maximum:
            description: Set maximum prefix limit
            type: dict
            suboptions:
              prefix:
                description:  Set table's maximum prefix limit.
                type: int
  running_config:
    description:
      - This option is used only with state I(parsed).
      - The value of this option should be the output received from the IOS-XR device by
        executing the command B(show running-config vrf).
      - The state I(parsed) reads the configuration from C(running_config) option and
        transforms it into Ansible structured data as per the resource module's argspec
        and the value is then returned in the I(parsed) key within the result.
    type: str
  state:
    choices: [parsed, gathered, deleted, merged, replaced, rendered, overridden]
    default: merged
    description:
      - The state the configuration should be left in
      - The states I(rendered), I(gathered) and I(parsed) does not perform any change
        on the device.
      - The state I(rendered) will transform the configuration in C(config) option to
        platform specific CLI commands which will be returned in the I(rendered) key
        within the result. For state I(rendered) active connection to remote host is
        not required.
      - The state I(gathered) will fetch the running configuration from device and transform
        it into structured data in the format as per the resource module argspec and
        the value is returned in the I(gathered) key within the result.
      - The state I(parsed) reads the configuration from C(running_config) option and
        transforms it into JSON format as per the resource module parameters and the
        value is returned in the I(parsed) key within the result. The value of C(running_config)
        option should be the same format as the output of command I(show running-config vrf).
        connection to remote host is not required.
    type: str
"""

EXAMPLES = """

# Using merged
#
# Before state:
# -------------
#
# RP/0/0/CPU0:iosxr#show running-config vrf
# vrf test
#

- name: Merge provided configuration with device configuration
  cisco.iosxr.iosxr_vrf_address_family:
    config:
      - name: VRF4
        address_families:
          - afi: "ipv4"
            safi: "unicast"
            export:
              route_target: "192.0.2.1:400"
              route_policy: "rm-policy"
              to:
                default_vrf:
                  route_policy: "rm-policy"
                vrf:
                  allow_imported_vpn: true
            import_config:
              route_target: "192.0.2.6:200"
              route_policy: "test-policy"
              from_config:
                bridge_domain:
                  advertise_as_vpn: true
                default_vrf:
                  route_policy: "test-policy"
                vrf:
                  advertise_as_vpn: true
            maximum:
              prefix: 100
    state: merged

# Task Output:
# ------------
#
# before: []
#
# commands:
# - vrf VRF4
# - address-family ipv4 unicast
# - export route-policy rm-policy
# - export route-target 192.0.2.1:400
# - export to default-vrf route-policy rm-policy
# - export to vrf allow-imported-vpn
# - import route-target 192.0.2.6:200
# - import route-policy test-policy
# - import from bridge-domain advertise-as-vpn
# - import from default-vrf route-policy test-policy
# - import from vrf advertise-as-vpn
# - maximum prefix 100
#
# after:
# - name: VRF4
#   address_families:
#     - afi: "ipv4"
#       safi: "unicast"
#       export:
#         route_target: "192.0.2.1:400"
#         route_policy: "rm-policy"
#         to:
#           default_vrf:
#             route_policy: "rm-policy"
#           vrf:
#             allow_imported_vpn: true
#       import_config:
#         route_target: "192.0.2.6:200"
#         route_policy: "test-policy"
#         from_config:
#           bridge_domain:
#             advertise_as_vpn: true
#           default_vrf:
#             route_policy: "test-policy"
#           vrf:
#             advertise_as_vpn: true
#       maximum:
#         prefix: 100
#
# After state:
# ------------
#
# RP/0/0/CPU0:iosxr#show running-config vrf
# vrf VRF4
#  address-family ipv4 unicast
#   export route-policy rm-policy
#   export route-target 192.0.2.1:400
#   export to default-vrf route-policy rm-policy
#   export to vrf allow-imported-vpn
#   import route-target 192.0.2.6:200
#   import route-policy test-policy
#   import from bridge-domain advertise-as-vpn
#   import from default-vrf route-policy test-policy
#   import from vrf advertise-as-vpn
#   maximum prefix 100

# Using replaced
#
# Before state:
# -------------
#
# RP/0/0/CPU0:iosxr#show running-config vrf
# vrf VRF4
#  address-family ipv4 unicast
#   export route-policy rm-policy
#   export route-target 192.0.2.1:400
#   export to default-vrf route-policy rm-policy
#   export to vrf allow-imported-vpn
#   import route-target 192.0.2.6:200
#   import route-policy test-policy
#   import from bridge-domain advertise-as-vpn
#   import from default-vrf route-policy test-policy
#   import from vrf advertise-as-vpn
#   maximum prefix 100

- name: Replace the provided configuration with the existing running configuration
  cisco.iosxr.iosxr_vrf_address_family:
    config:
      - name: VRF7
        address_families:
          - afi: "ipv4"
            safi: "unicast"
            export:
              route_target: "192.0.2.2:400"
              route_policy: "rm-policy"
              to:
                default_vrf:
                  route_policy: "rm-policy"
                vrf:
                  allow_imported_vpn: true
            import_config:
              route_target: "192.0.2.4:400"
              route_policy: "test-policy"
              from_config:
                bridge_domain:
                  advertise_as_vpn: true
                default_vrf:
                  route_policy: "test-policy"
                vrf:
                  advertise_as_vpn: true
            maximum:
              prefix: 200
    state: replaced

# Task Output:
# ------------
#
# - name: VRF4
#   address_families:
#     - afi: "ipv4"
#       safi: "unicast"
#       export:
#         route_target: "192.0.2.1:400"
#         route_policy: "rm-policy"
#         to:
#           default_vrf:
#             route_policy: "rm-policy"
#           vrf:
#             allow_imported_vpn: true
#       import_config:
#         route_target: "192.0.2.6:200"
#         route_policy: "test-policy"
#         from_config:
#           bridge_domain:
#             advertise_as_vpn: true
#           default_vrf:
#             route_policy: "test-policy"
#           vrf:
#             advertise_as_vpn: true
#       maximum:
#         prefix: 100
#
# commands:
# - vrf VRF7
# - address-family ipv4 unicast
# - export route-policy rm-policy
# - export route-target 192.0.2.2:400
# - export to default-vrf route-policy rm-policy
# - export to vrf allow-imported-vpn
# - import route-target 192.0.2.4:400
# - import route-policy test-policy
# - import from bridge-domain advertise-as-vpn
# - import from default-vrf route-policy test-policy
# - import from vrf advertise-as-vpn
# - maximum prefix 200
#
# after:
# - name: VRF7
#   address_families:
#     - afi: "ipv4"
#       safi: "unicast"
#       export:
#         route_target: "192.0.2.2:400"
#         route_policy: "rm-policy"
#         to:
#           default_vrf:
#             route_policy: "rm-policy"
#           vrf:
#             allow_imported_vpn: true
#       import_config:
#         route_target: "192.0.2.4:400"
#         route_policy: "test-policy"
#         from_config:
#           bridge_domain:
#             advertise_as_vpn: true
#           default_vrf:
#             route_policy: "test-policy"
#           vrf:
#             advertise_as_vpn: true
#       maximum:
#         prefix: 200
#
# After state:
# ------------
#
# RP/0/RP0/CPU0:iosxr(config)#show running-config vrf
# vrf VRF7
#  address-family ipv4 unicast
#   import route-policy test-policy
#   import from bridge-domain advertise-as-vpn
#   import from default-vrf route-policy test-policy
#   import from vrf advertise-as-vpn
#   import route-target
#    192.0.2.4:400
#   !
#   export route-policy rm-policy
#   export to vrf allow-imported-vpn
#   export to default-vrf route-policy rm-policy
#   export route-target
#    192.0.2.2:400
#   !
#   maximum prefix 200

# Using overridden
#
# Before state:
# -------------
#
# RP/0/RP0/CPU0:iosxr(config)#show running-config vrf
# vrf VRF7
#  address-family ipv4 unicast
#   import route-policy test-policy
#   import from bridge-domain advertise-as-vpn
#   import from default-vrf route-policy test-policy
#   import from vrf advertise-as-vpn
#   import route-target
#    192.0.2.4:400
#   !
#   export route-policy rm-policy
#   export to vrf allow-imported-vpn
#   export to default-vrf route-policy rm-policy
#   export route-target
#    192.0.2.2:400
#   !
#   maximum prefix 200

- name: Override the provided configuration with the existing running configuration
  cisco.iosxr.iosxr_vrf_address_family:
    state: overridden
    config:
      - name: VRF6
        address_families:
          - afi: "ipv4"
            safi: "unicast"
            export:
              route_target: "192.0.2.8:200"
              route_policy: "rm-policy1"
              to:
                default_vrf:
                  route_policy: "rm-policy"
                vrf:
                  allow_imported_vpn: "true"
            import_config:
              route_target: "192.0.2.2:200"
              route_policy: "test-policy"
              from_config:
                bridge_domain:
                  advertise_as_vpn: "true"
                default_vrf:
                  route_policy: "test-policy"
                vrf:
                  advertise_as_vpn: "true"
            maximum:
              prefix: 500
# Task Output:
# ------------
#
# before:
# - name: VRF7
#   address_families:
#     - afi: "ipv4"
#       safi: "unicast"
#       export:
#         route_target: "192.0.2.2:400"
#         route_policy: "rm-policy"
#         to:
#           default_vrf:
#             route_policy: "rm-policy"
#           vrf:
#             allow_imported_vpn: true
#       import_config:
#         route_target: "192.0.2.4:400"
#         route_policy: "test-policy"
#         from_config:
#           bridge_domain:
#             advertise_as_vpn: true
#           default_vrf:
#             route_policy: "test-policy"
#           vrf:
#             advertise_as_vpn: true
#       maximum:
#         prefix: 200
#
# commands:
# - vrf VRF7
# - address-family ipv4 unicast
# - no import route-policy test-policy
# - no import from bridge-domain advertise-as-vpn
# - no import from default-vrf route-policy test-policy
# - no import from vrf advertise-as-vpn
# - no import route-target 192.0.2.4:400
# - no export route-policy rm-policy
# - no export route-target 192.0.2.2:400
# - no export to default-vrf route-policy rm-policy
# - no export to vrf allow-imported-vpn
# - no maximum prefix 200
# - vrf VRF6
# - address-family ipv4 unicast
# - export route-policy rm-policy1
# - export route-target 192.0.2.8:200
# - export to default-vrf route-policy rm-policy
# - export to vrf allow-imported-vpn
# - import route-target 192.0.2.2:200
# - import route-policy test-policy
# - import from bridge-domain advertise-as-vpn
# - import from default-vrf route-policy test-policy
# - import from vrf advertise-as-vpn
# - maximum prefix 500
#
# after:
# - name: VRF4
# - name: VRF6
#   address_families:
#     - afi: "ipv4"
#       safi: "unicast"
#       export:
#         route_target: "192.0.2.8:200"
#         route_policy: "rm-policy1"
#         to:
#           default_vrf:
#             route_policy: "rm-policy"
#           vrf:
#             allow_imported_vpn: "true"
#       import_config:
#         route_target: "192.0.2.2:200"
#         route_policy: "test-policy"
#         from_config:
#           bridge_domain:
#             advertise_as_vpn: "true"
#           default_vrf:
#             route_policy: "test-policy"
#           vrf:
#             advertise_as_vpn: "true"
#       maximum:
#         prefix: 500
# - name: VRF7
#
# After state:
# -------------
# RP/0/RP0/CPU0:iosxr(config)#show running-config vrf
# vrf VRF4
# vrf VRF6
#  address-family ipv4 unicast
#   import route-policy test-policy
#   import from bridge-domain advertise-as-vpn
#   import from default-vrf route-policy test-policy
#   import from vrf advertise-as-vpn
#   import route-target
#    192.0.2.2:200
#   export route-policy rm-policy1
#   export to vrf allow-imported-vpn
#   export to default-vrf route-policy rm-policy
#   export route-target
#    192.0.2.8:200
#   maximum prefix 500
# vrf VRF7

# Using deleted
#
# Before state:
# -------------
#
# RP/0/RP0/CPU0:iosxr(config)#show running-config vrf
# vrf VRF4
# vrf VRF6
#  address-family ipv4 unicast
#   import route-policy test-policy
#   import from bridge-domain advertise-as-vpn
#   import from default-vrf route-policy test-policy
#   import from vrf advertise-as-vpn
#   import route-target
#    192.0.2.2:200
#   export route-policy rm-policy1
#   export to vrf allow-imported-vpn
#   export to default-vrf route-policy rm-policy
#   export route-target
#    192.0.2.8:200
#   maximum prefix 500
# vrf VRF7

- name: Delete the provided configuration
  cisco.iosxr.iosxr_vrf_address_family:
    config:
    state: deleted

# Task Output:
# ------------
#
# before:
# - name: VRF4
# - name: VRF6
#   address_families:
#     - afi: "ipv4"
#       safi: "unicast"
#       export:
#         route_target: "192.0.2.8:200"
#         route_policy: "rm-policy1"
#         to:
#           default_vrf:
#             route_policy: "rm-policy"
#           vrf:
#             allow_imported_vpn: "true"
#       import_config:
#         route_target: "192.0.2.2:200"
#         route_policy: "test-policy"
#         from_config:
#           bridge_domain:
#             advertise_as_vpn: "true"
#           default_vrf:
#             route_policy: "test-policy"
#           vrf:
#             advertise_as_vpn: "true"
#       maximum:
#         prefix: 500
# - name: VRF7

# commands:
# - vrf VRF4
# - vrf VRF6
# - no address-family ipv4 unicast
# - vrf VRF7
#
# after:
# - name: VRF4
# - name: VRF6
# - name: VRF7
#
# After state:
# ------------
#
# RP/0/RP0/CPU0:iosxr(config)#show running-config vrf
# vrf VRF4
# vrf VRF6
# vrf VRF7

# Using rendered
#
- name: Render provided configuration with device configuration
  cisco.iosxr.iosxr_vrf_address_family:
    config:
      - name: VRF4
        address_families:
          - afi: "ipv4"
            safi: "unicast"
            export:
              route_target: "192.0.2.1:400"
              route_policy: "rm-policy"
              to:
                default_vrf:
                  route_policy: "rm-policy"
                vrf:
                  allow_imported_vpn: true
            import_config:
              route_target: "192.0.2.6:200"
              route_policy: "test-policy"
              from_config:
                bridge_domain:
                  advertise_as_vpn: true
                default_vrf:
                  route_policy: "test-policy"
                vrf:
                  advertise_as_vpn: true
            maximum:
              prefix: 100
    state: rendered

# Task Output:
# ------------
#
# rendered:
# - vrf VRF4
# - address-family ipv4 unicast
# - export route-policy rm-policy
# - export route-target 192.0.2.1:400
# - export to default-vrf route-policy rm-policy
# - export to vrf allow-imported-vpn
# - import route-target 192.0.2.6:200
# - import route-policy test-policy
# - import from bridge-domain advertise-as-vpn
# - import from default-vrf route-policy test-policy
# - import from vrf advertise-as-vpn
# - maximum prefix 100

# Using gathered
#
# Before state:
# -------------
#
# RP/0/RP0/CPU0:iosxr(config)#show running-config vrf
# vrf VRF4
#  address-family ipv4 unicast
#   export route-policy rm-policy
#   export route-target 192.0.2.1:400
#   export to default-vrf route-policy rm-policy
#   export to vrf allow-imported-vpn
#   import route-target 192.0.2.6:200
#   import route-policy test-policy
#   import from bridge-domain advertise-as-vpn
#   import from default-vrf route-policy test-policy
#   import from vrf advertise-as-vpn
#   maximum prefix 100

- name: Gather existing running configuration
  cisco.iosxr.iosxr_vrf_address_family:
    state: gathered

# Task Output:
# ------------
#
# gathered:
# - name: VRF4
#   address_families:
#     - afi: "ipv4"
#       safi: "unicast"
#       export:
#         route_target: "192.0.2.1:400"
#         route_policy: "rm-policy"
#         to:
#           default_vrf:
#             route_policy: "rm-policy"
#           vrf:
#             allow_imported_vpn: true
#       import_config:
#         route_target: "192.0.2.6:200"
#         route_policy: "test-policy"
#         from_config:
#           bridge_domain:
#             advertise_as_vpn: true
#           default_vrf:
#             route_policy: "test-policy"
#           vrf:
#             advertise_as_vpn: true
#       maximum:
#         prefix: 100

# Using parsed
#
# File: parsed.cfg
# ----------------
#
# vrf test
#  address-family ipv4 unicast
#   export to default-vrf route-policy "rm-policy"
#   export to vrf allow-imported-vpn
#   export route-policy "export-policy"
#   export route-target
#    192.0.2.1:400
#   import route-target
#    192.0.2.2:200
#   import route-policy "test-policy"
#   import from bridge-domain advertise-as-vpn
#   import from default-vrf route-policy "new-policy"
#   import from vrf advertise-as-vpn
#   maximum prefix 23

- name: Parse the provided configuration
  cisco.iosxr.iosxr_vrf_address_family:
    running_config: "{{ lookup('file', 'parsed.cfg') }}"
    state: parsed

# Task Output:
# ------------
#
# parsed:
#   - address_families:
#     - afi: ipv4
#       export:
#         route_policy: export-policy
#         route_target: 192.0.2.1:400
#         to:
#           default_vrf:
#             route_policy: rm-policy
#           vrf:
#             allow_imported_vpn: true
#       import_config:
#         from_config:
#           bridge_domain:
#             advertise_as_vpn: true
#           default_vrf:
#             route_policy: new-policy
#           vrf:
#             advertise_as_vpn: true
#         route_policy: test-policy
#         route_target: 192.0.2.2:200
#       maximum:
#         prefix: 23
#       safi: unicast
#     name: test
"""

RETURN = """
before:
  description: The configuration prior to the model invocation.
  returned: always
  type: list
  sample: >
    The configuration returned will always be in the same format
     of the parameters above.
after:
  description: The resulting configuration model invocation.
  returned: when changed
  type: list
  sample: >
    The configuration returned will always be in the same format
     of the parameters above.
commands:
  description: The set of commands pushed to the remote device.
  returned: always
  type: list
  sample:
  - vrf VRF7
  - address-family ipv4 unicast
  - export route-policy rm-policy
  - import route-policy test-policy
rendered:
  description: The provided configuration in the task rendered in device-native format (offline).
  returned: when I(state) is C(rendered)
  type: list
  sample:
  - vrf VRF4
  - address-family ipv4 unicast
  - export route-policy rm-policy
gathered:
  description: Facts about the network resource gathered from the remote device as structured data.
  returned: when I(state) is C(gathered)
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
parsed:
  description: The device native config provided in I(running_config) option parsed into structured data as per module argspec.
  returned: when I(state) is C(parsed)
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.argspec.vrf_address_family.vrf_address_family import (
    Vrf_address_familyArgs,
)
from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.config.vrf_address_family.vrf_address_family import (
    Vrf_address_family,
)


def main():
    """
    Main entry point for module execution

    :returns: the result form module invocation
    """
    module = AnsibleModule(
        argument_spec=Vrf_address_familyArgs.argument_spec,
        mutually_exclusive=[["config", "running_config"]],
        required_if=[
            ["state", "merged", ["config"]],
            ["state", "replaced", ["config"]],
            ["state", "overridden", ["config"]],
            ["state", "rendered", ["config"]],
            ["state", "parsed", ["running_config"]],
        ],
        supports_check_mode=True,
    )

    result = Vrf_address_family(module).execute_module()
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2023, Pedro Nascimento <apecnascimento@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: nomad_token
author: Pedro Nascimento (@apecnascimento)
version_added: "8.1.0"
short_description: Manage Nomad ACL tokens
description:
  - This module allows to create Bootstrap tokens, create ACL tokens, update ACL tokens, and delete ACL tokens.
requirements:
  - python-nomad
extends_documentation_fragment:
  - community.general._nomad
  - community.general._attributes
attributes:
  check_mode:
    support: none
  diff_mode:
    support: none
options:
  name:
    description:
      - Name of ACL token to create.
    type: str
  token_type:
    description:
      - The type of the token can be V(client), V(management), or V(bootstrap).
    choices: ["client", "management", "bootstrap"]
    type: str
    default: "client"
  policies:
    description:
      - A list of the policies assigned to the token.
    type: list
    elements: str
    default: []
  global_replicated:
    description:
      - Indicates whether or not the token was created with the C(--global).
    type: bool
    default: false
  state:
    description:
      - Create or remove ACL token.
    choices: ["present", "absent"]
    required: true
    type: str

seealso:
  - name: Nomad ACL documentation
    description: Complete documentation for Nomad API ACL.
    link: https://developer.hashicorp.com/nomad/api-docs/acl/tokens
"""

EXAMPLES = r"""
- name: Create boostrap token
  community.general.nomad_token:
    host: localhost
    token_type: bootstrap
    state: present

- name: Create ACL token
  community.general.nomad_token:
    host: localhost
    name: "Dev token"
    token_type: client
    policies:
      - readonly
    global_replicated: false
    state: absent

- name: Update ACL token Dev token
  community.general.nomad_token:
    host: localhost
    name: "Dev token"
    token_type: client
    policies:
      - readonly
      - devpolicy
    global_replicated: false
    state: absent

- name: Delete ACL token
  community.general.nomad_token:
    host: localhost
    name: "Dev token"
    state: absent
"""

RETURN = r"""
result:
  description: Result returned by nomad.
  returned: always
  type: dict
  sample:
    {
      "accessor_id": "0d01c55f-8d63-f832-04ff-1866d4eb594e",
      "create_index": 14,
      "create_time": "2023-11-12T18:48:34.248857001Z",
      "expiration_time": null,
      "expiration_ttl": "",
      "global": true,
      "hash": "eSn8H8RVqh8As8WQNnC2vlBRqXy6DECogc5umzX0P30=",
      "modify_index": 836,
      "name": "devs",
      "policies": [
        "readonly"
      ],
      "roles": null,
      "secret_id": "12e878ab-e1f6-e103-b4c4-3b5173bb4cea",
      "type": "client"
    }
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._nomad import argument_spec as nomad_argument_spec
from ansible_collections.community.general.plugins.module_utils._nomad import nomad, setup_nomad_client


def get_token(name, nomad_client):
    tokens = nomad_client.acl.get_tokens()
    token = next((token for token in tokens if token.get("Name") == name), None)
    return token


def transform_response(nomad_response):
    transformed_response = {
        "accessor_id": nomad_response["AccessorID"],
        "create_index": nomad_response["CreateIndex"],
        "create_time": nomad_response["CreateTime"],
        "expiration_ttl": nomad_response["ExpirationTTL"],
        "expiration_time": nomad_response["ExpirationTime"],
        "global": nomad_response["Global"],
        "hash": nomad_response["Hash"],
        "modify_index": nomad_response["ModifyIndex"],
        "name": nomad_response["Name"],
        "policies": nomad_response["Policies"],
        "roles": nomad_response["Roles"],
        "secret_id": nomad_response["SecretID"],
        "type": nomad_response["Type"],
    }

    return transformed_response


argument_spec = {
    **nomad_argument_spec,
    "state": dict(required=True, choices=["present", "absent"]),
    "name": dict(type="str"),
    "token_type": dict(choices=["client", "management", "bootstrap"], default="client"),
    "policies": dict(type="list", elements="str", default=[]),
    "global_replicated": dict(type="bool", default=False),
}


def setup_module_object():
    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=False,
        required_one_of=[["name", "token_type"]],
        required_if=[
            ("token_type", "client", ("name",)),
            ("token_type", "management", ("name",)),
        ],
    )
    return module


def run(module):
    nomad_client = setup_nomad_client(module)

    msg = ""
    result = {}
    changed = False
    if module.params["state"] == "present":
        if module.params["token_type"] == "bootstrap":
            try:
                current_token = get_token("Bootstrap Token", nomad_client)
                if current_token:
                    msg = "ACL bootstrap already exist."
                else:
                    nomad_result = nomad_client.acl.generate_bootstrap()
                    msg = "Boostrap token created."
                    result = transform_response(nomad_result)
                    changed = True

            except nomad.api.exceptions.URLNotAuthorizedNomadException:
                try:
                    nomad_result = nomad_client.acl.generate_bootstrap()
                    msg = "Boostrap token created."
                    result = transform_response(nomad_result)
                    changed = True

                except Exception as e:
                    module.fail_json(msg=f"{e}")
        else:
            try:
                token_info = {
                    "Name": module.params["name"],
                    "Type": module.params["token_type"],
                    "Policies": module.params["policies"],
                    "Global": module.params["global_replicated"],
                }

                current_token = get_token(token_info["Name"], nomad_client)

                if current_token:
                    token_info["AccessorID"] = current_token["AccessorID"]
                    nomad_result = nomad_client.acl.update_token(current_token["AccessorID"], token_info)
                    msg = "ACL token updated."
                    result = transform_response(nomad_result)
                    changed = True

                else:
                    nomad_result = nomad_client.acl.create_token(token_info)
                    msg = "ACL token Created."
                    result = transform_response(nomad_result)
                    changed = True

            except Exception as e:
                module.fail_json(msg=f"{e}")

    if module.params["state"] == "absent":
        if not module.params["name"]:
            module.fail_json(msg="name is needed to delete token.")

        if module.params["token_type"] == "bootstrap" or module.params["name"] == "Bootstrap Token":
            module.fail_json(msg="Delete ACL bootstrap token is not allowed.")

        try:
            token = get_token(module.params["name"], nomad_client)
            if token:
                nomad_client.acl.delete_token(token.get("AccessorID"))
                msg = "ACL token deleted."
                changed = True
            else:
                msg = f"No token with name '{module.params['name']}' found"

        except Exception as e:
            module.fail_json(msg=f"{e}")

    module.exit_json(changed=changed, msg=msg, result=result)


def main():
    module = setup_module_object()
    run(module)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: autoscaling_complete_lifecycle_action
short_description: Completes the lifecycle action of an instance
description:
  - Used to complete the lifecycle action for the specified instance with the specified result.
version_added: "4.1.0"
author:
  - Saleh Abbas (@salehabbas) <saleh.abbas@thetradedesk.com>
options:
  asg_name:
    description:
      - The name of the Auto Scaling Group which the instance belongs to.
    type: str
    required: true
  lifecycle_hook_name:
    description:
      - The name of the lifecycle hook to complete.
    type: str
    required: true
  lifecycle_action_result:
    description:
      - The action for the lifecycle hook to take.
    choices: ['CONTINUE', 'ABANDON']
    type: str
    required: true
  instance_id:
    description:
      - The ID of the instance.
    type: str
    required: true
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.
# Complete the lifecycle action
- aws_asg_complete_lifecycle_action:
    asg_name: my-auto-scaling-group
    lifecycle_hook_name: my-lifecycle-hook
    lifecycle_action_result: CONTINUE
    instance_id: i-123knm1l2312
"""

RETURN = r"""
---
status:
    description: How things went
    returned: success
    type: str
    sample: ["OK"]
"""

try:
    import botocore
except ImportError:
    pass  # caught by AnsibleAWSModule

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule


def main():
    argument_spec = dict(
        asg_name=dict(required=True, type="str"),
        lifecycle_hook_name=dict(required=True, type="str"),
        lifecycle_action_result=dict(required=True, type="str", choices=["CONTINUE", "ABANDON"]),
        instance_id=dict(required=True, type="str"),
    )

    module = AnsibleAWSModule(argument_spec=argument_spec)

    asg_name = module.params.get("asg_name")
    lifecycle_hook_name = module.params.get("lifecycle_hook_name")
    lifecycle_action_result = module.params.get("lifecycle_action_result")
    instance_id = module.params.get("instance_id")

    autoscaling = module.client("autoscaling")
    try:
        results = autoscaling.complete_lifecycle_action(
            LifecycleHookName=lifecycle_hook_name,
            AutoScalingGroupName=asg_name,
            LifecycleActionResult=lifecycle_action_result,
            InstanceId=instance_id,
        )
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to completes the lifecycle action")

    module.exit_json(results=results)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: networkfirewall_policy
short_description: manage AWS Network Firewall policies
version_added: 4.0.0
description:
  - A module for creating, updating and deleting AWS Network Firewall policies.
options:
  arn:
    description:
      - The ARN of the Network Firewall policy.
      - Exactly one of I(arn) or I(name) must be provided.
    required: false
    type: str
  name:
    description:
      - The name of the Network Firewall policy.
      - Cannot be updated after creation.
      - Exactly one of I(arn) or I(name) must be provided.
    required: false
    type: str
  state:
    description:
      - Create or remove the Network Firewall policy.
    required: false
    choices: ['present', 'absent']
    default: 'present'
    type: str
  description:
    description:
      - A description for the Network Firewall policy.
    required: false
    type: str
  stateful_rule_groups:
    description:
      - A list of names or ARNs of stateful firewall rule groups.
    required: false
    type: list
    elements: str
    aliases: ['stateful_groups']
  stateless_rule_groups:
    description:
      - A list of names or ARNs of stateless firewall rule groups.
    required: false
    type: list
    elements: str
    aliases: ['stateless_groups']
  stateless_default_actions:
    description:
      - Actions to take on a packet if it doesn't match any of the stateless
        rules in the policy.
      - Common actions are C(aws:pass), C(aws:drop) and C(aws:forward_to_sfe).
      - When creating a new policy defaults to C(aws:forward_to_sfe).
    required: false
    type: list
    elements: str
  stateless_fragment_default_actions:
    description:
      - Actions to take on a fragmented UDP packet if it doesn't match any
        of the stateless rules in the policy.
      - Common actions are C(aws:pass), C(aws:drop) and C(aws:forward_to_sfe).
      - When creating a new policy defaults to C(aws:forward_to_sfe).
    required: false
    type: list
    elements: str
  stateful_default_actions:
    description:
      - Actions to take on a packet if it doesn't match any of the stateful
        rules in the policy.
      - Common actions are C(aws:drop_strict), C(aws:drop_established),
        C(aws:alert_strict) and C(aws:alert_established).
      - Only valid for policies where I(strict_rule_order=true).
      - When creating a new policy defaults to C(aws:drop_strict).
    required: false
    type: list
    elements: str
  stateful_rule_order:
    description:
      - Indicates how to manage the order of stateful rule evaluation for the policy.
      - When I(strict_rule_order='strict') rules and rule groups are evaluated in
        the order that they're defined.
      - Cannot be updated after creation.
    required: false
    type: str
    choices: ['default', 'strict']
    aliases: ['rule_order']
  stateless_custom_actions:
    description:
      - A list of dictionaries defining custom actions which can be used in
        I(stateless_default_actions) and I(stateless_fragment_default_actions).
    required: false
    type: list
    elements: dict
    aliases: ['custom_stateless_actions']
    suboptions:
      name:
        description:
          - The name of the custom action.
        required: true
        type: str
      publish_metric_dimension_value:
        description:
          - When the custom action is used, metrics will have a dimension of
            C(CustomAction) the value of which is set to
            I(publish_metric_dimension_value).
        required: false
        type: str
        aliases: ['publish_metric_dimension_values']
  purge_stateless_custom_actions:
    description:
      - If I(purge_stateless_custom_actions=true), existing custom actions will
        be purged from the resource to match exactly what is defined by
        the I(stateless_custom_actions) parameter.
    type: bool
    required: false
    default: True
    aliases: ['purge_custom_stateless_actions']
  wait:
    description:
      - Whether to wait for the firewall policy to reach the
        C(ACTIVE) or C(DELETED) state before the module returns.
    type: bool
    required: false
    default: true
  wait_timeout:
    description:
      - Maximum time, in seconds, to wait for the firewall policy
        to reach the expected state.
      - Defaults to 600 seconds.
    type: int
    required: false

author:
  - Mark Chappell (@tremble)
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
  - amazon.aws.tags
"""

EXAMPLES = r"""
# Create an AWS Network Firewall Policy with default rule order
- community.aws.networkfirewall_policy:
    stateful_rule_order: 'default'
    state: present
    name: 'ExamplePolicy'

# Create an AWS Network Firewall Policy with strict rule order
- community.aws.networkfirewall_policy:
    stateful_rule_order: 'strict'
    state: present
    name: 'ExampleStrictPolicy'


# Create an AWS Network Firewall Policy that defaults to dropping all packets
- community.aws.networkfirewall_policy:
    stateful_rule_order: 'strict'
    state: present
    name: 'ExampleDropPolicy'
    stateful_default_actions:
      - 'aws:drop_strict'
    stateful_rule_groups:
      - 'ExampleStrictRuleGroup'
      - 'arn:aws:network-firewall:us-east-1:aws-managed:stateful-rulegroup/BotNetCommandAndControlDomainsStrictOrder'

# Delete an AWS Network Firewall Policy
- community.aws.networkfirewall_policy:
    state: absent
    name: 'ExampleDropPolicy'
"""

RETURN = r"""
policy:
  description: The details of the policy
  type: dict
  returned: success
  contains:
    policy:
      description: The details of the policy
      type: dict
      returned: success
      contains:
        stateful_engine_options:
          description:
            - Extra options describing how the stateful rules should be handled.
          type: dict
          returned: success
          contains:
            rule_order:
              description:
                - How rule group evaluation will be ordered.
                - For more information on rule evaluation ordering see the AWS documentation
                  U(https://docs.aws.amazon.com/network-firewall/latest/developerguide/suricata-rule-evaluation-order.html).
              type: str
              returned: success
              example: 'DEFAULT_ACTION_ORDER'
        stateful_rule_group_references:
          description: Information about the stateful rule groups attached to the policy.
          type: list
          elements: dict
          returned: success
          contains:
            resource_arn:
              description: The ARN of the rule group.
              type: str
              returned: success
              example: 'arn:aws:network-firewall:us-east-1:aws-managed:stateful-rulegroup/AbusedLegitMalwareDomainsActionOrder'
            priority:
              description:
                - An integer that indicates the order in which to run the stateful rule groups in a single policy.
                - This only applies to policies that specify the STRICT_ORDER rule order in the stateful engine options settings.
              type: int
              returned: success
              example: 1234
        stateless_custom_actions:
          description:
            - A description of additional custom actions available for use as
              default rules to apply to stateless packets.
          type: list
          elements: dict
          returned: success
          contains:
            action_name:
              description: A name for the action.
              type: str
              returned: success
              example: 'ExampleAction'
            action_definition:
              description: The action to perform.
              type: dict
              returned: success
              contains:
                publish_metric_action:
                  description:
                    - Definition of a custom metric to be published to CloudWatch.
                    - U(https://docs.aws.amazon.com/network-firewall/latest/developerguide/monitoring-cloudwatch.html)
                  type: dict
                  returned: success
                  contains:
                    dimensions:
                      description:
                        - The values of the CustomAction dimension to set on the metrics.
                        - The dimensions of a metric are used to identify unique
                          streams of data.
                      type: list
                      elements: dict
                      returned: success
                      contains:
                        value:
                          description: A value of the CustomAction dimension to set on the metrics.
                          type: str
                          returned: success
                          example: 'ExampleRule'
        stateless_default_actions:
          description: The default actions to take on a packet that doesn't match any stateful rules.
          type: list
          elements: str
          returned: success
          example: ['aws:alert_strict']
        stateless_fragment_default_actions:
          description: The actions to take on a packet if it doesn't match any of the stateless rules in the policy.
          type: list
          elements: str
          returned: success
          example: ['aws:pass']
        stateless_rule_group_references:
          description: Information about the stateful rule groups attached to the policy.
          type: list
          elements: dict
          returned: success
          contains:
            resource_arn:
              description: The ARN of the rule group.
              type: str
              returned: success
              example: 'arn:aws:network-firewall:us-east-1:123456789012:stateless-rulegroup/ExampleGroup'
            priority:
              description:
                - An integer that indicates the order in which to run the stateless rule groups in a single policy.
              type: str
              returned: success
              example: 12345
    policy_metadata:
      description: Metadata about the policy
      type: dict
      returned: success
      contains:
        consumed_stateful_rule_capacity:
          description: The total number of capacity units used by the stateful rule groups.
          type: int
          returned: success
          example: 165
        consumed_stateless_rule_capacity:
          description: The total number of capacity units used by the stateless rule groups.
          type: int
          returned: success
          example: 2010
        firewall_policy_arn:
          description: The ARN of the policy.
          type: str
          returned: success
          example: arn:aws:network-firewall:us-east-1:123456789012:firewall-policy/ExamplePolicy
        firewall_policy_id:
          description: The unique ID of the policy.
          type: str
          returned: success
          example: 12345678-abcd-1234-5678-123456789abc
        firewall_policy_name:
          description: The name of the policy.
          type: str
          returned: success
          example: ExamplePolicy
        firewall_policy_status:
          description: The current status of the policy.
          type: str
          returned: success
          example: ACTIVE
        number_of_associations:
          description: The number of firewalls the policy is associated to.
          type: int
          returned: success
          example: 1
        tags:
          description: A dictionary representing the tags associated with the policy.
          type: dict
          returned: success
          example: {'tagName': 'Some Value'}
"""

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule
from ansible_collections.community.aws.plugins.module_utils.networkfirewall import NetworkFirewallPolicyManager


def main():
    custom_action_options = dict(
        name=dict(type="str", required=True),
        # Poorly documented, but "publishMetricAction.dimensions ... must have length less than or equal to 1"
        publish_metric_dimension_value=dict(type="str", required=False, aliases=["publish_metric_dimension_values"]),
        # NetworkFirewallPolicyManager can cope with a list for future-proofing
        # publish_metric_dimension_values=dict(type='list', elements='str', required=False, aliases=['publish_metric_dimension_value']),
    )

    argument_spec = dict(
        name=dict(type="str", required=False),
        arn=dict(type="str", required=False),
        state=dict(type="str", required=False, default="present", choices=["present", "absent"]),
        description=dict(type="str", required=False),
        tags=dict(type="dict", required=False, aliases=["resource_tags"]),
        purge_tags=dict(type="bool", required=False, default=True),
        stateful_rule_groups=dict(type="list", elements="str", required=False, aliases=["stateful_groups"]),
        stateless_rule_groups=dict(type="list", elements="str", required=False, aliases=["stateless_groups"]),
        stateful_default_actions=dict(type="list", elements="str", required=False),
        stateless_default_actions=dict(type="list", elements="str", required=False),
        stateless_fragment_default_actions=dict(type="list", elements="str", required=False),
        stateful_rule_order=dict(type="str", required=False, choices=["strict", "default"], aliases=["rule_order"]),
        stateless_custom_actions=dict(
            type="list",
            elements="dict",
            required=False,
            options=custom_action_options,
            aliases=["custom_stateless_actions"],
        ),
        purge_stateless_custom_actions=dict(
            type="bool", required=False, default=True, aliases=["purge_custom_stateless_actions"]
        ),
        wait=dict(type="bool", required=False, default=True),
        wait_timeout=dict(type="int", required=False),
    )

    mutually_exclusive = [
        ["arn", "name"],
    ]
    required_one_of = [
        ["arn", "name"],
    ]

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        mutually_exclusive=mutually_exclusive,
        required_one_of=required_one_of,
    )

    arn = module.params.get("arn")
    name = module.params.get("name")
    state = module.params.get("state")

    manager = NetworkFirewallPolicyManager(module, name=name, arn=arn)
    manager.set_wait(module.params.get("wait", None))
    manager.set_wait_timeout(module.params.get("wait_timeout", None))

    if state == "absent":
        manager.delete()
    else:
        manager.set_description(module.params.get("description", None))
        manager.set_tags(module.params.get("tags", None), module.params.get("purge_tags", None))
        # Actions need to be defined before potentially consuming them
        manager.set_custom_stateless_actions(
            module.params.get("stateless_custom_actions", None),
            module.params.get("purge_stateless_custom_actions", True),
        )
        manager.set_stateful_rule_order(module.params.get("stateful_rule_order", None))
        manager.set_stateful_rule_groups(module.params.get("stateful_rule_groups", None))
        manager.set_stateless_rule_groups(module.params.get("stateless_rule_groups", None))
        manager.set_stateful_default_actions(module.params.get("stateful_default_actions", None))
        manager.set_stateless_default_actions(module.params.get("stateless_default_actions", None))
        manager.set_stateless_fragment_default_actions(module.params.get("stateless_fragment_default_actions", None))

        manager.flush_changes()

    results = dict(
        changed=manager.changed,
        policy=manager.updated_resource,
    )
    if manager.changed:
        diff = dict(
            before=manager.original_resource,
            after=manager.updated_resource,
        )
        results["diff"] = diff
    module.exit_json(**results)


if __name__ == "__main__":
    main()



# Copyright (c) 2024 Bill Peck  <bpeck@redhat.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import os

from ansible.errors import AnsibleActionFail
from ansible.plugins.action import ActionBase
from ansible.utils.display import Display
from ansible_collections.azure.azcollection.plugins.plugin_utils import (file_utils, ssh_info, connectivity_utils)
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_rest import GenericRestClient
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMAuth
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AZURE_COMMON_ARGS
try:
    from azure.core.exceptions import ClientAuthenticationError, ResourceExistsError, ResourceNotFoundError
except ImportError:
    pass


display = Display()


class ActionModule(ActionBase):
    ''' Configures ssh proxy for connecting to ARC hosts '''

    TRANSFERS_FILES = False
    BYPASS_HOST_LOOP = True

    def __init__(self, *args, **kwargs):
        if not connectivity_utils.HAS_ORAS:
            raise AnsibleActionFail("oras.client is not installed.", orig_exc=connectivity_utils.HAS_ORAS_EXC)
        super(ActionModule, self).__init__(*args, **kwargs)

    def run(self, tmp=None, task_vars=None):
        ''' run the pause action module '''
        if task_vars is None:
            task_vars = dict()

        result = super(ActionModule, self).run(tmp, task_vars)
        del tmp  # tmp no longer has any effect

        merged_arg_spec = dict()
        merged_arg_spec.update(AZURE_COMMON_ARGS)
        merged_arg_spec.update(
            {
                'inventory_hostname': {'type': 'str'},
                'ansible_host': {'type': 'str'},
                'resource_group': {'type': 'str'},
                'resource_type': {'type': 'str'},
                'private_key_file': {'type': 'str'},
                'local_user': {'type': 'str'},
                'port': {'type': 'int'},
                'ssh_config_file': {'type': 'str'},
                'ssh_relay_file': {'type': 'str'},
                'ssh_proxy_folder': {'type': 'str'}
            }
        )

        validation_result, new_module_args = self.validate_argument_spec(
            argument_spec=merged_arg_spec,
        )

        auth_source = os.environ.get('ANSIBLE_AZURE_AUTH_SOURCE', None) or new_module_args.get('auth_source')
        auth_options = dict(
            auth_source=auth_source,
            profile=new_module_args.get('profile'),
            subscription_id=new_module_args.get('subscription_id'),
            client_id=new_module_args.get('client_id'),
            secret=new_module_args.get('secret'),
            tenant=new_module_args.get('tenant'),
            ad_user=new_module_args.get('ad_user'),
            password=new_module_args.get('password'),
            cloud_environment=new_module_args.get('cloud_environment'),
            cert_validation_mode=new_module_args.get('cert_validation_mode'),
            api_profile=new_module_args.get('api_profile'),
            track1_cred=True,
            adfs_authority_url=new_module_args.get('adfs_authority_url')
        )

        inventory_hostname = new_module_args.get('inventory_hostname')
        ansible_host = new_module_args.get('ansible_host')
        resource_group = new_module_args.get('resource_group')
        resource_type = new_module_args.get('resource_type')
        private_key_file = new_module_args.get('private_key_file')
        local_user = new_module_args.get('local_user')
        port = new_module_args.get('port')
        ssh_config_file = new_module_args.get('ssh_config_file')
        ssh_relay_file = new_module_args.get('ssh_relay_file')
        ssh_proxy_folder = new_module_args.get('ssh_proxy_folder')
        result.update(dict(
            changed=False,
            rc=0,
            stderr='',
            stdout=''
        ))

        ########################################################################
        # Begin the hard work!

        azure_auth = AzureRMAuth(**auth_options)
        rest_client = GenericRestClient(azure_auth.azure_credential_track2,
                                        azure_auth.subscription_id,
                                        azure_auth._cloud_environment.endpoints.resource_manager,
                                        credential_scopes=[azure_auth._cloud_environment.endpoints.resource_manager + ".default"])
        # Define error_map with common http error codes
        rest_client.error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
        }

        config_session = ssh_info.ConfigSession(ssh_config_file,
                                                ssh_relay_file,
                                                resource_group,
                                                inventory_hostname,
                                                ansible_host,
                                                private_key_file,
                                                local_user,
                                                port,
                                                resource_type,
                                                ssh_proxy_folder)

        try:
            cert_lifetime = None  # If set to None we default to the max which is 1 hour
            config_session.proxy_path = connectivity_utils.install_client_side_proxy(config_session.ssh_proxy_folder)
            (config_session.relay_info,
             config_session.new_service_config) = connectivity_utils.get_relay_information(rest_client,
                                                                                           azure_auth.subscription_id,
                                                                                           config_session.resource_group_name,
                                                                                           config_session.hostname,
                                                                                           config_session.resource_type,
                                                                                           cert_lifetime,
                                                                                           config_session.port)
        except Exception as e:
            raise AnsibleActionFail("Failed to get relay information.", orig_exc=e)

        config_text = config_session.get_config_text()
        ssh_config_path = config_session.ssh_config_file

        ssh_config_dir = os.path.dirname(ssh_config_path)
        if not os.path.isdir(ssh_config_dir):
            os.makedirs(ssh_config_dir)

        file_utils.write_to_file(ssh_config_path,
                                 'w',
                                 '\n'.join(config_text),
                                 f"Couldn't write ssh config to file {ssh_config_path}.",
                                 'utf-8')

        result['stdout'] = "SSH proxy configured for %s in %s" % (inventory_hostname, config_session.ssh_config_file)
        return result



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2025 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
The module file for ios_bfd_templates
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
module: ios_bfd_templates
extends_documentation_fragment:
  - cisco.ios.ios
short_description: Bidirectional Forwarding Detection (BFD) templates configurations
description:
   - Manages Bidirectional Forwarding Detection (BFD) templates configurations
version_added: 11.3.0
author:
  - Komal Desai (@komaldesai13)
notes:
  - Tested against Cisco IOS XE Software, Version 17.13.01a on CML
  - This module works with connection C(network_cli)
options:
  config:
    description: A dictionary of bfd template options
    type: list
    elements: dict
    suboptions:
      name:
        description: name of the BFD template to be used
        required: true
        type: str
      hop:
        description: type of template to be used
        required: true
        choices:
          - single_hop
          - multi_hop
        type: str
      interval:
        description: defines transmit interval between BFD packets
        type: dict
        suboptions:
          min_tx:
            description: The minimum interval in milliseconds that the local system desires for transmitting BFD control packets
            type: int
          min_rx:
            description: The minimum interval in milliseconds that the local system is capable of supporting between received BFD control packets
            type: int
          multiplier:
            description: Specifies the number of consecutive BFD control packets that must be missed from a BFD peer before BFD declares\
                         that the peer is unavailable and the Layer 3 BFD peer is informed of the failure.
            type: int
      dampening:
        description: enables session dampening
        type: dict
        suboptions:
          half_life_period:
            description: half-life period for the exponential decay algorithm, in minutes.
            type: int
          reuse_threshold:
            description: The threshold at which a dampened session is allowed to be reused (taken out of dampening), in milliseconds.
            type: int
          suppress_threshold:
            description: The threshold at which a session is suppressed (put into dampening), in milliseconds.
            type: int
          max_suppress_time:
            description: The maximum amount of time a session can be suppressed, in minutes.
            type: int
      echo:
        description: enables the BFD echo function for all interfaces which uses this specific template.
        type: bool
      authentication:
        description: Configure authentication for the BFD template
        type: dict
        suboptions:
          type:
            description: Authentication type to use for BFD sessions
            type: str
            choices:
              - sha_1
              - md5
              - meticulous_md5
              - meticulous_sha_1
          keychain:
            description: Name of the key chain to use for authentication
            type: str
  running_config:
    description:
      - This option is used only with state I(parsed).
      - The value of this option should be the output received from the IOS device by
        executing the command B(show running-config | section ^bfd-template).
      - The state I(parsed) reads the configuration from C(running_config) option and
        transforms it into Ansible structured data as per the resource module's argspec
        and the value is then returned in the I(parsed) key within the result.
    type: str
  state:
    choices:
      - merged
      - replaced
      - overridden
      - deleted
      - rendered
      - gathered
      - purged
      - parsed
    default: merged
    description:
      - The state the configuration should be left in
      - The states I(rendered), I(gathered) and I(parsed) does not perform any change
        on the device.
      - The state I(rendered) will transform the configuration in C(config) option to
        platform specific CLI commands which will be returned in the I(rendered) key
        within the result. For state I(rendered) active connection to remote host is
        not required.
      - The state I(gathered) will fetch the running configuration from device and transform
        it into structured data in the format as per the resource module argspec and
        the value is returned in the I(gathered) key within the result.
      - The state I(parsed) reads the configuration from C(running_config) option and
        transforms it into JSON format as per the resource module parameters and the
        value is returned in the I(parsed) key within the result. The value of C(running_config)
        option should be the same format as the output of command I(show running-config
        | section ^bfd-template) executed on device. For state I(parsed) active
        connection to remote host is not required.
      - The state C(deleted) does not support or guarantee granular deletion of configuration
        the playbook should act as source of truth, and the desired state of the resouce is what
        the playbook should reflect. Use C(overridden) or C(replaced) to get extra configuration
        removed.
      - The state I(purged) removes BFD templates completely using a single top-level
        C(no bfd-template) command, which removes the entire template definition at once.
    type: str
"""

EXAMPLES = """
# Using merged
# Before state:
# -------------
# router-ios#show running-config | section ^bfd-template
# (no BFD templates configured)

- name: Merge provided configuration with device configuration
  cisco.ios.ios_bfd_templates:
    config:
      - name: template1
        hop: single_hop
        interval:
          min_tx: 200
          min_rx: 200
          multiplier: 3
        authentication:
          type: sha_1
          keychain: bfd_keychain
        echo: true
      - name: template2
        hop: multi_hop
        interval:
          min_tx: 500
          min_rx: 500
          multiplier: 5
        dampening:
          half_life_period: 30
          reuse_threshold: 2000
          suppress_threshold: 5000
          max_suppress_time: 120
    state: merged

# Commands Fired:
# ---------------
# bfd-template single-hop template1
#  interval min-tx 200 min-rx 200 multiplier 3
#  authentication sha-1 keychain bfd_keychain
#  echo
# bfd-template multi-hop template2
#  interval min-tx 500 min-rx 500 multiplier 5
#  dampening 30 2000 5000 120

# After state:
# ------------
# router-ios#show running-config | section ^bfd-template
# bfd-template single-hop template1
#  interval min-tx 200 min-rx 200 multiplier 3
#  authentication sha-1 keychain bfd_keychain
#  echo
# bfd-template multi-hop template2
#  interval min-tx 500 min-rx 500 multiplier 5
#  dampening 30 2000 5000 120

# Using replaced
# Before state:
# -------------
# router-ios#show running-config | section ^bfd-template
# bfd-template single-hop template1
#  interval min-tx 200 min-rx 200 multiplier 3
#  authentication sha-1 keychain bfd_keychain
#  echo
# bfd-template multi-hop template2
#  interval min-tx 500 min-rx 500 multiplier 5

- name: Replace device configuration of specified BFD templates with provided configuration
  cisco.ios.ios_bfd_templates:
    config:
      - name: template1
        hop: single_hop
        interval:
          min_tx: 300
          min_rx: 300
          multiplier: 4
        authentication:
          type: sha_1
          keychain: new_keychain
    state: replaced

# Commands Fired:
# ---------------
# bfd-template single-hop template1
#  no echo
#  interval min-tx 300 min-rx 300 multiplier 4
#  no authentication sha-1 keychain bfd_keychain
#  authentication sha-1 keychain new_keychain

# After state:
# ------------
# router-ios#show running-config | section ^bfd-template
# bfd-template single-hop template1
#  interval min-tx 300 min-rx 300 multiplier 4
#  authentication sha-1 keychain new_keychain
# bfd-template multi-hop template2
#  interval min-tx 500 min-rx 500 multiplier 5

# Using overridden
# Before state:
# -------------
# router-ios#show running-config | section ^bfd-template
# bfd-template single-hop template1
#  interval min-tx 200 min-rx 200 multiplier 3
# bfd-template multi-hop template2
#  interval min-tx 500 min-rx 500 multiplier 5
# bfd-template single-hop template3
#  echo

- name: Override device configuration with provided configuration
  cisco.ios.ios_bfd_templates:
    config:
      - name: template1
        hop: single_hop
        interval:
          min_tx: 300
          min_rx: 300
          multiplier: 5
        authentication:
          type: md5
          keychain: secure_key
    state: overridden

# Commands Fired:
# ---------------
# no bfd-template multi-hop template2
# no bfd-template single-hop template3
# bfd-template single-hop template1
#  interval min-tx 300 min-rx 300 multiplier 5
#  authentication md5 keychain secure_key

# After state:
# ------------
# router-ios#show running-config | section ^bfd-template
# bfd-template single-hop template1
#  interval min-tx 300 min-rx 300 multiplier 5
#  authentication md5 keychain secure_key

# Using deleted
# Before state:
# -------------
# router-ios#show running-config | section ^bfd-template
# bfd-template single-hop template1
#  interval min-tx 200 min-rx 200 multiplier 3
#  authentication sha-1 keychain bfd_keychain
#  echo
# bfd-template multi-hop template2
#  interval min-tx 500 min-rx 500 multiplier 5

- name: Delete specified BFD template
  cisco.ios.ios_bfd_templates:
    config:
      - name: template1
        hop: single_hop
    state: deleted

# Commands Fired:
# ---------------
# bfd-template single-hop template1
#  no echo
#  no interval min-tx 200 min-rx 200 multiplier 3
#  no authentication sha-1 keychain bfd_keychain

# After state:
# ------------
# router-ios#show running-config | section ^bfd-template
# bfd-template single-hop template1
# bfd-template multi-hop template2
#  interval min-tx 500 min-rx 500 multiplier 5

# Using deleted (to delete all BFD templates )
# Before state:
# -------------
# router-ios#show running-config | section ^bfd-template
# bfd-template single-hop template1
#  interval min-tx 200 min-rx 200 multiplier 3
# bfd-template multi-hop template2
#  interval min-tx 500 min-rx 500 multiplier 5

- name: Delete all BFD template configurations
  cisco.ios.ios_bfd_templates:
    state: deleted

# Commands Fired:
# ---------------
# bfd-template single-hop template1
#  no interval min-tx 200 min-rx 200 multiplier 3
# bfd-template multi-hop template2
#  no interval min-tx 500 min-rx 500 multiplier 5

# After state:
# ------------
# router-ios#show running-config | section ^bfd-template
# bfd-template single-hop template1
# bfd-template multi-hop template2

# Using purged
# Before state:
# -------------
# router-ios#show running-config | section ^bfd-template
# bfd-template single-hop template1
#  interval min-tx 200 min-rx 200 multiplier 3
# bfd-template multi-hop template2
#  interval min-tx 500 min-rx 500 multiplier 5

- name: Purge specified BFD template configurations (complete removal)
  cisco.ios.ios_bfd_templates:
    config:
      - name: template1
        hop: single_hop
    state: purged

# Commands Fired:
# ---------------
# no bfd-template single-hop template1

# After state:
# ------------
# router-ios#show running-config | section ^bfd-template
# bfd-template multi-hop template2
#  interval min-tx 500 min-rx 500 multiplier 5

# Using purged (to remove all BFD templates)
# Before state:
# -------------
# router-ios#show running-config | section ^bfd-template
# bfd-template single-hop template1
#  interval min-tx 200 min-rx 200 multiplier 3
#  authentication sha-1 keychain bfd_keychain
# bfd-template multi-hop template2
#  interval min-tx 500 min-rx 500 multiplier 5
# bfd-template single-hop template3
#  echo

- name: Purge all BFD template configurations (complete removal)
  cisco.ios.ios_bfd_templates:
    config: []
    state: purged

# Commands Fired:
# ---------------
# no bfd-template single-hop template1
# no bfd-template multi-hop template2
# no bfd-template single-hop template3

# After state:
# ------------
# router-ios#show running-config | section ^bfd-template
# (no BFD templates configured)

# Using rendered
- name: Render platform specific commands from task input using rendered state
  cisco.ios.ios_bfd_templates:
    config:
      - name: template1
        hop: single_hop
        interval:
          min_tx: 200
          min_rx: 200
          multiplier: 3
        authentication:
          type: meticulous_sha_1
          keychain: secure_chain
        echo: true
    state: rendered

# Module Execution Result:
# ------------------------
# "rendered": [
#     "bfd-template single-hop template1",
#     "interval min-tx 200 min-rx 200 multiplier 3",
#     "authentication meticulous-sha-1 keychain secure_chain",
#     "echo"
# ]

# Using gathered
- name: Gather BFD template configuration from the device
  cisco.ios.ios_bfd_templates:
    state: gathered

# Module Execution Result:
# ------------------------
# "gathered": [
#     {
#         "name": "template1",
#         "hop": "single_hop",
#         "interval": {
#             "min_tx": 200,
#             "min_rx": 200,
#             "multiplier": 3
#         },
#         "authentication": {
#             "type": "sha_1",
#             "keychain": "bfd_keychain"
#         },
#         "echo": true
#     }
# ]

# Using parsed
- name: Parse the provided configuration to structured format
  cisco.ios.ios_bfd_templates:
    running_config: |
      bfd-template single-hop template1
       interval min-tx 200 min-rx 200 multiplier 3
       authentication sha-1 keychain bfd_keychain
       echo
      bfd-template multi-hop template2
       dampening 30 2000 5000 120
    state: parsed

# Module Execution Result:
# ------------------------
# "parsed": [
#     {
#         "name": "template1",
#         "hop": "single_hop",
#         "interval": {
#             "min_tx": 200,
#             "min_rx": 200,
#             "multiplier": 3
#         },
#         "authentication": {
#             "type": "sha_1",
#             "keychain": "bfd_keychain"
#         },
#         "echo": true
#     },
#     {
#         "name": "template2",
#         "hop": "multi_hop",
#         "dampening": {
#             "half_life_period": 30,
#             "reuse_threshold": 2000,
#             "suppress_threshold": 5000,
#             "max_suppress_time": 120
#         }
#     }
# ]
"""

RETURN = """
before:
  description: The configuration prior to the module invocation.
  returned: always
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
after:
  description: The resulting configuration after module execution.
  returned: when changed
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
commands:
  description: The set of commands pushed to the remote device.
  returned: always
  type: list
  sample:
    - bfd-template single-hop template1
    - interval min-tx 200 min-rx 200 multiplier 3
    - authentication sha-1 keychain bfd_keychain
    - echo
rendered:
  description: The provided configuration in the task rendered in device native format (offline).
  returned: when I(state) is C(rendered)
  type: list
  sample:
    - bfd-template single-hop template1
    - interval min-tx 200 min-rx 200 multiplier 3
    - authentication sha-1 keychain bfd_keychain
    - echo
gathered:
  description: Facts about the network resource gathered from the remote device as structured data.
  returned: when I(state) is C(gathered)
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
parsed:
  description: The device native config provided in I(running_config) option parsed into structured data as per module argspec.
  returned: when I(state) is C(parsed)
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
"""


from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.ios.plugins.module_utils.network.ios.argspec.bfd_templates.bfd_templates import (
    Bfd_templatesArgs,
)
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.config.bfd_templates.bfd_templates import (
    Bfd_templates,
)


def main():
    """
    Main entry point for module execution

    :returns: the result form module invocation
    """
    module = AnsibleModule(
        argument_spec=Bfd_templatesArgs.argument_spec,
        mutually_exclusive=[["config", "running_config"]],
        required_if=[
            ["state", "merged", ["config"]],
            ["state", "replaced", ["config"]],
            ["state", "overridden", ["config"]],
            ["state", "rendered", ["config"]],
            ["state", "parsed", ["running_config"]],
        ],
        supports_check_mode=True,
    )

    result = Bfd_templates(module).execute_module()
    module.exit_json(**result)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
"""
The eos lacp fact class
It is in this file the configuration is collected from the device
for a given resource, parsed, and the facts tree is populated
based on the configuration.
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

import re

from copy import deepcopy

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common import utils

from ansible_collections.arista.eos.plugins.module_utils.network.eos.argspec.lacp.lacp import (
    LacpArgs,
)


class LacpFacts(object):
    """The eos lacp fact class"""

    def __init__(self, module, subspec="config", options="options"):
        self._module = module
        self.argument_spec = LacpArgs.argument_spec
        spec = deepcopy(self.argument_spec)
        if subspec:
            if options:
                facts_argument_spec = spec[subspec][options]
            else:
                facts_argument_spec = spec[subspec]
        else:
            facts_argument_spec = spec

        self.generated_spec = utils.generate_dict(facts_argument_spec)

    def get_device_data(self, connection):
        return connection.get("show running-config | section ^lacp")

    def populate_facts(self, connection, ansible_facts, data=None):
        """Populate the facts for lacp
        :param connection: the device connection
        :param ansible_facts: Facts dictionary
        :param data: previously collected configuration
        :rtype: dictionary
        :returns: facts
        """
        if not data:
            data = self.get_device_data(connection)

        # split the config into instances of the resource
        resource_delim = "lacp"
        find_pattern = r"(?:^|\n)%s.*?(?=(?:^|\n)%s|$)" % (
            resource_delim,
            resource_delim,
        )
        resources = [p.strip() for p in re.findall(find_pattern, data, re.DOTALL)]

        objs = {}
        for resource in resources:
            if resource:
                obj = self.render_config(self.generated_spec, resource)
                if obj:
                    objs.update(obj)

        ansible_facts["ansible_network_resources"].pop("lacp", None)
        facts = {"lacp": {}}
        if objs:
            params = utils.validate_config(
                self.argument_spec,
                {"config": objs},
            )
            facts["lacp"] = utils.remove_empties(params["config"])

        ansible_facts["ansible_network_resources"].update(facts)
        return ansible_facts

    def render_config(self, spec, conf):
        """
        Render config as dictionary structure and delete keys
          from spec for null values

        :param spec: The facts tree, generated from the argspec
        :param conf: The configuration
        :rtype: dictionary
        :returns: The generated config
        """
        config = deepcopy(spec)
        config["system"]["priority"] = utils.parse_conf_arg(
            conf,
            "system-priority",
        )

        return utils.remove_empties(config)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2025, Gaspard Micol (@gmicol) <gmicol@cisco.com>

# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["preview"],
    "supported_by": "community",
}

DOCUMENTATION = r"""
---
module: ndo_qos_class_policy
version_added: "2.11.0"
short_description: Manage QoS Class Policies on Cisco Nexus Dashboard Orchestrator (NDO).
description:
- Manage Quality of Service (QoS) Class Policies.
- This module can be used on Cisco Nexus Dashboard Orchestrator (NDO).
- There can only be a single QoS Class policy in a fabric policy template.
- This module is only supported on ND v3.1 (NDO v4.3) and later.
author:
- Gaspard Micol (@gmicol)
options:
  template:
    description:
    - The name of the template.
    - The template must be a fabric policy template.
    - This parameter or O(template_id) is required.
    type: str
  template_id:
    description:
    - The ID of the fabric policy template.
    - This parameter or O(template) is required.
    type: str
  name:
    description:
    - The name of the QoS Class Policy.
    type: str
    aliases: [ qos_class_policy ]
  uuid:
    description:
    - The UUID of the QoS Class Policy.
    type: str
    aliases: [ qos_class_policy_uuid ]
  description:
    description:
    - The description of the QoS Class Policy.
    type: str
  preserve_cos:
    description:
    - Whether to preserve the Class of Service (CoS).
    - Defaults to true when unset during creation.
    type: bool
  qos_levels:
    description:
    - The list of configurable QoS levels for the QoS Class Policy.
    - Providing a new list of O(qos_levels) will replace the existing one from the QoS Class Policy.
    - Providing an empty list will remove the O(qos_levels=[]) from the QoS Class Policy.
    type: list
    elements: dict
    suboptions:
      level:
        description:
        - The QoS level.
        type: str
        choices: [ level1, level2, level3, level4, level5, level6 ]
        required: true
      mtu:
        description:
        - The MTU value.
        - The value must be between 1500 and 9216.
        - Defaults to 9216 when unset during creation.
        type: int
      minimum_buffer:
        description:
        - The minimum number of reserved buffers.
        - The value must be between 0 and 3.
        - Defaults to 0 when unset during creation.
        type: int
      congestion_algorithm:
        description:
        - The congestion algorithm used for this QoS Level.
        - Defaults to C(tail_drop) when unset during creation.
        type: str
        choices: [ tail_drop, wred ]
      wred_configuration:
        description:
        - The Weighted Random Early Detection (WRED) Algorithm configuration.
        - Providing a new list of O(qos_levels.wred_configuration) will replace the existing one from the list of QoS levels.
        - Providing an empty list will remove the O(qos_levels.wred_configuration=[]) from the list of QoS levels.
          from the QoS Class Policy.
        type: dict
        suboptions:
          congestion_notification:
            description:
            - The state of Explicit Congestion Notification (ECN) setting.
            - Enabling Congestion Notification causes the packets that would be dropped to be ECN-marked instead.
            - Defaults to C(disabled) when unset during creation.
            type: str
            choices: [ enabled, disabled ]
          forward_non_ecn_traffic:
            description:
            - Whether to forward Non-ECN Traffic.
            - This attribute should only be used when O(qos_levels.wred_configuration.congestion_notification="enabled").
            - Defaults to false when unset during creation.
            type: bool
          minimum_threshold:
            description:
            - The minimum queue threshold as a percentage of the maximum queue length for WRED algorithm.
            - The value must be between 0 and 100.
            - Defaults to 0 when unset during creation.
            type: int
          maximum_threshold:
            description:
            - The maximum queue threshold as a percentage of the maximum queue length for WRED algorithm.
            - The value must be between 0 and 100.
            - Defaults to 100 when unset during creation.
            type: int
          probability:
            description:
            - The probability value for WRED algorithm.
            - The probability used to determine whether a packet is dropped or queued
              when the average queue size is between the minimum and the maximum threshold values.
            - The value must be between 0 and 100.
            - Defaults to 0 when unset during creation.
            type: int
          weight:
            description:
            - The weight value for WRED algorithm.
            - Lower weight prioritizes current queue length, while higher weight prioritizes older queue lengths.
            - The value must be between 0 and 7.
            - Defaults to 0 when unset during creation.
            type: int
      scheduling_algorithm:
        description:
        - The QoS Scheduling Algorithm.
        - Defaults to C(weighted_round_robin) when unset during creation.
        type: str
        choices: [ weighted_round_robin, strict_priority ]
      bandwidth_allocated:
        description:
        - The percentage of total bandwidth allocated to this QoS Level.
        - The value must be between 0 and 100.
        - Defaults to 20 when unset during creation.
        type: int
      pfc_admin_state:
        description:
        - The administrative state of the Priority Flow Control (PFC) policy.
        type: str
        choices: [ enabled, disabled ]
        default: disabled
      admin_state:
        description:
        - The policy administrative state.
        - Defaults to C(enabled) when unset during creation.
        type: str
        choices: [ enabled, disabled ]
      no_drop_cos:
        description:
        - The Class of Service (CoS) level for which to enforce the no drop packet handling even in case of traffic congestion.
        - This attribute must be specified when O(qos_levels.pfc_admin_state="enabled").
        - Defaults to C(unspecified) when unset during creation.
        type: str
        choices: [ cos0, cos1, cos2, cos3, cos4, cos5, cos6, cos7, unspecified ]
      pfc_scope:
        description:
        - The PFC scope.
        - Defaults to C(fabric_wide) when unset during creation.
        type: str
        choices: [ fabric_wide, intra_tor ]
  state:
    description:
    - Use C(absent) for removing.
    - Use C(query) for listing an object or multiple objects.
    - Use C(present) for creating or updating.
    type: str
    choices: [ absent, query, present ]
    default: query
notes:
- The O(template) must exist before using this module in your playbook.
  The M(cisco.mso.ndo_template) module can be used for this.
- Attempts to create any additional QoS Class policies will only update the existing
  object in the Fabric Policy template.
seealso:
extends_documentation_fragment: cisco.mso.modules
"""

EXAMPLES = r"""
- name: Create a new QoS Class policy with minimum configuration
  cisco.mso.ndo_qos_class_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    name: ansible_test_qos_class_policy
    state: present
  register: create_qos_class_policy

- name: Update a QoS Class policy using UUID by adding QoS level1 with minimum configuration
  cisco.mso.ndo_qos_class_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    uuid: "{{ create_qos_class_policy.current.uuid }}"
    description: Ansible Test QoS Class Policy
    preserve_cos: true
    qos_levels:
      - level: level1
    state: present
  register: update_qos_class_policy_with_qos_level1

- name: Update a QoS Class policy by adding QoS level2 with full configuration
  cisco.mso.ndo_qos_class_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    name: ansible_test_qos_class_policy
    description: Ansible Test QoS Class Policy
    preserve_cos: true
    qos_levels:
      - level: level1
      - level: level2
        mtu: 9000
        minimum_buffer: 1
        congestion_algorithm: wred
        wred_configuration:
          congestion_notification: enabled
          forward_non_ecn_traffic: false
          minimum_threshold: 5
          maximum_threshold: 95
          probability: 80
          weight: 1
        scheduling_algorithm: weighted_round_robin
        bandwidth_allocated: 50
        pfc_admin_state: enabled
        admin_state: enabled
        no_drop_cos: cos1
        pfc_scope: intra_tor
    state: present
  register: add_qos_class_policy_level2

- name: Update a QoS Class policy by removing QoS level2 and keeping QoS level1
  cisco.mso.ndo_qos_class_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    name: ansible_test_qos_class_policy
    description: Ansible Test QoS Class Policy
    preserve_cos: true
    qos_levels:
      - level: level1
    state: present
  register: remove_qos_class_policy_level2

- name: Update a QoS Class policy by removing all QoS levels
  cisco.mso.ndo_qos_class_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    name: ansible_test_qos_class_policy
    description: Ansible Test QoS Class Policy
    preserve_cos: true
    qos_levels: []
    state: present
  register: remove_qos_class_policy_all_levels

- name: Query QoS Class policy using name
  cisco.mso.ndo_qos_class_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    name: ansible_test_qos_class_policy
    state: query
  register: query_one

- name: Query QoS Class policy using UUID
  cisco.mso.ndo_qos_class_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    uuid: "{{ create_qos_class_policy.current.uuid }}"
    state: query
  register: query_one_uuid

- name: Delete a QoS Class policy using name
  cisco.mso.ndo_qos_class_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    name: ansible_test_qos_class_policy
    state: absent

- name: Delete a QoS Class policy using UUID
  cisco.mso.ndo_qos_class_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    uuid: "{{ query_one.current.uuid }}"
    state: absent
"""

RETURN = r"""
"""
import copy
from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import MSOModule, mso_argument_spec
from ansible_collections.cisco.mso.plugins.module_utils.template import MSOTemplate, KVPair
from ansible_collections.cisco.mso.plugins.module_utils.utils import append_update_ops_data, delete_none_values
from ansible_collections.cisco.mso.plugins.module_utils.constants import (
    QOS_CONGESTION_ALGORITHM_MAP,
    QOS_SCHEDULING_ALGORITHM_MAP,
    QOS_PFC_SCOPE_MAP,
    QOS_LEVEL,
    COS_VALUES,
)


def main():
    qos_levels_to_remove = copy.copy(QOS_LEVEL)  # A list that tracks of all levels to be removed during the PATCH operation when qos_levels is being updated
    qos_levels_to_remove.remove("unspecified")  # "unspecified" is removed as it is not a valid value for qos_levels.level
    argument_spec = mso_argument_spec()
    argument_spec.update(
        template=dict(type="str"),
        template_id=dict(type="str"),
        name=dict(type="str", aliases=["qos_class_policy"]),
        description=dict(type="str"),
        uuid=dict(type="str", aliases=["qos_class_policy_uuid"]),
        preserve_cos=dict(type="bool"),
        qos_levels=dict(
            type="list",
            elements="dict",
            options=dict(
                level=dict(type="str", required=True, choices=qos_levels_to_remove),
                mtu=dict(type="int"),
                minimum_buffer=dict(type="int"),
                congestion_algorithm=dict(type="str", choices=list(QOS_CONGESTION_ALGORITHM_MAP)),
                wred_configuration=dict(
                    type="dict",
                    options=dict(
                        congestion_notification=dict(type="str", choices=["enabled", "disabled"]),
                        forward_non_ecn_traffic=dict(type="bool"),
                        minimum_threshold=dict(type="int"),
                        maximum_threshold=dict(type="int"),
                        probability=dict(type="int"),
                        weight=dict(type="int"),
                    ),
                ),
                scheduling_algorithm=dict(type="str", choices=list(QOS_SCHEDULING_ALGORITHM_MAP)),
                bandwidth_allocated=dict(type="int"),
                pfc_admin_state=dict(type="str", choices=["enabled", "disabled"], default="disabled"),
                admin_state=dict(type="str", choices=["enabled", "disabled"]),
                no_drop_cos=dict(type="str", choices=COS_VALUES),
                pfc_scope=dict(type="str", choices=list(QOS_PFC_SCOPE_MAP)),
            ),
            required_if=[
                ["pfc_admin_state", "enabled", ["no_drop_cos"]],
            ],
        ),
        state=dict(type="str", default="query", choices=["absent", "query", "present"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["name", "uuid"], True],
            ["state", "present", ["name", "uuid"], True],
        ],
        required_one_of=[
            ["template", "template_id"],
        ],
        mutually_exclusive=[
            ["template", "template_id"],
        ],
    )

    mso = MSOModule(module)

    template = module.params.get("template")
    template_id = module.params.get("template_id")
    name = module.params.get("name")
    description = module.params.get("description")
    uuid = module.params.get("uuid")
    preserve_cos = module.params.get("preserve_cos")
    qos_levels = module.params.get("qos_levels")
    state = module.params.get("state")

    ops = []
    match = None

    mso_template = MSOTemplate(mso, "fabric_policy", template, template_id)
    mso_template.validate_template("fabricPolicy")
    object_description = "QoS Class Policy"

    path = "/fabricPolicyTemplate/template/qosClass"
    existing_qos_policies = mso_template.template.get("fabricPolicyTemplate", {}).get("template", {}).get("qosClass", {})
    if name or uuid:
        match = mso_template.get_object_by_key_value_pairs(
            object_description,
            [existing_qos_policies],
            [KVPair("uuid", uuid) if uuid else KVPair("name", name)],
        )
        if match:
            mso.existing = mso.previous = copy.deepcopy(mso_template.update_config_with_template_and_references(match.details))
    else:
        mso.existing = mso.previous = mso_template.update_config_with_template_and_references(existing_qos_policies)

    if state == "present":
        if uuid and not mso.existing:
            mso.fail_json(msg="{0} with the UUID: '{1}' not found".format(object_description, uuid))

        mso_values = {
            "name": name,
            "description": description,
            "preserveCos": preserve_cos,
        }
        if qos_levels is not None:
            for qos_level in qos_levels:
                level = qos_level.get("level")
                if level not in qos_levels_to_remove:
                    mso.fail_json(msg="Duplicate configurations for QoS {0}".format(level))
                else:
                    qos_levels_to_remove.remove(level)

                wred_configuration = None
                if qos_level.get("wred_configuration"):
                    wred_configuration = {
                        "congestionNotification": qos_level["wred_configuration"].get("congestion_notification"),
                        "minThreshold": qos_level["wred_configuration"].get("minimum_threshold"),
                        "maxThreshold": qos_level["wred_configuration"].get("maximum_threshold"),
                        "probability": qos_level["wred_configuration"].get("probability"),
                        "weight": qos_level["wred_configuration"].get("weight"),
                        "forwardNonEcn": qos_level["wred_configuration"].get("forward_non_ecn_traffic"),
                    }

                mso_values[level] = {
                    "adminState": qos_level.get("admin_state"),
                    "minBuffer": qos_level.get("minimum_buffer"),
                    "mtu": qos_level.get("mtu"),
                    "congestionAlgorithm": QOS_CONGESTION_ALGORITHM_MAP.get(qos_level.get("congestion_algorithm")),
                    "wredConfig": wred_configuration,
                    "schedulingAlgorithm": QOS_SCHEDULING_ALGORITHM_MAP.get(qos_level.get("scheduling_algorithm")),
                    "bandwidthAllocated": qos_level.get("bandwidth_allocated"),
                    "pfcAdminState": qos_level.get("pfc_admin_state"),
                    "noDropCoS": qos_level.get("no_drop_cos"),
                    "pfcScope": QOS_PFC_SCOPE_MAP.get(qos_level.get("pfc_scope")),
                }
        else:
            qos_levels_to_remove = []

        mso_values = delete_none_values(mso_values)

        if match:
            append_update_ops_data(ops, match.details, path, mso_values, remove_data=qos_levels_to_remove)
            mso.sanitize(match.details, collate=True)

        else:
            mso.sanitize(mso_values)
            ops.append(dict(op="add", path=path, value=mso.sent))

    elif state == "absent":
        if match:
            ops.append(dict(op="remove", path=path))

    if not module.check_mode and ops:
        response = mso.request(mso_template.template_path, method="PATCH", data=ops)
        qos_policies = response.get("fabricPolicyTemplate", {}).get("template", {}).get("qosClass", {})
        match = mso_template.get_object_by_key_value_pairs(
            object_description,
            [qos_policies],
            [KVPair("uuid", uuid) if uuid else KVPair("name", name)],
        )
        if match:
            mso.existing = mso_template.update_config_with_template_and_references(match.details)
        else:
            mso.existing = {}
    elif module.check_mode and state != "query":
        mso_template.update_config_with_template_and_references(mso.proposed)
        mso.existing = mso.proposed if state == "present" else {}

    mso.exit_json()


if __name__ == "__main__":
    main()



# Copyright (c) 2018, Scott Buchanan <sbuchanan@ri.pn>
# Copyright (c) 2016, Andrew Zenk <azenk@umn.edu> (lastpass.py used as starting point)
# Copyright (c) 2018, Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
name: onepassword_raw
author:
  - Scott Buchanan (@scottsb)
  - Andrew Zenk (@azenk)
  - Sam Doran (@samdoran)
short_description: Fetch an entire item from 1Password
description:
  - P(community.general.onepassword_raw#lookup) wraps C(op) command line utility to fetch an entire item from 1Password.
options:
  _terms:
    description: Identifier(s) (case-insensitive UUID or name) of item(s) to retrieve.
    required: true
    type: list
    elements: string
  account_id:
    version_added: 7.5.0
  domain:
    version_added: 6.0.0
  service_account_token:
    version_added: 7.1.0
extends_documentation_fragment:
  - community.general._onepassword
  - community.general._onepassword.lookup
"""

EXAMPLES = r"""
---
- name: Retrieve all data about Wintermute
  ansible.builtin.debug:
    var: lookup('community.general.onepassword_raw', 'Wintermute')

- name: Retrieve all data about Wintermute when not signed in to 1Password
  ansible.builtin.debug:
    var: lookup('community.general.onepassword_raw', 'Wintermute', subdomain='Turing', vault_password='DmbslfLvasjdl')
"""

RETURN = r"""
_raw:
  description: Entire item requested.
  type: list
  elements: dict
"""

import json

from ansible.plugins.lookup import LookupBase

from ansible_collections.community.general.plugins.lookup.onepassword import OnePass
from ansible_collections.community.general.plugins.plugin_utils._lookup import check_for_wrong_terms


class LookupModule(LookupBase):
    def run(self, terms, variables=None, **kwargs):
        self.set_options(var_options=variables, direct=kwargs)
        check_for_wrong_terms(self, direct=kwargs)

        vault = self.get_option("vault")
        subdomain = self.get_option("subdomain")
        domain = self.get_option("domain", "1password.com")
        username = self.get_option("username")
        secret_key = self.get_option("secret_key")
        master_password = self.get_option("master_password")
        service_account_token = self.get_option("service_account_token")
        account_id = self.get_option("account_id")
        connect_host = self.get_option("connect_host")
        connect_token = self.get_option("connect_token")

        op = OnePass(
            subdomain=subdomain,
            domain=domain,
            username=username,
            secret_key=secret_key,
            master_password=master_password,
            service_account_token=service_account_token,
            account_id=account_id,
            connect_host=connect_host,
            connect_token=connect_token,
        )
        op.assert_logged_in()

        values = []
        for term in terms:
            data = json.loads(op.get_raw(term, vault))
            values.append(data)

        return values
